from __future__ import annotations
from typing import Optional, Union

from PySide6.QtCore import QPointF, Qt, QRectF
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout,
    QGraphicsItem
)
from PySide6.QtCharts import (
    QChart, QChartView, QSplineSeries,
    QBarCategoryAxis, QValueAxis,
    QScatterSeries, QLineSeries
)
from PySide6.QtGui import (
    QPen, QPainter, QColor, QFont, QBrush, QFontMetrics
)

from qtmui.material.styles import useTheme


# -----------------------------------------------------
# TooltipItem (marker circle + text + border)
# -----------------------------------------------------
class TooltipItem(QGraphicsItem):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.lines = []          # [(text, QColor)]
        self.padding = 8
        self.bg_color = QColor(20, 20, 20, 230)
        self.border_color = QColor(255, 255, 255, 90)
        self.border_radius = 6
        self.font = QFont("Segoe UI", 9)

        self._w = 0
        self._h = 0
        self.setZValue(99999)
        self.hide()

    def clear(self):
        self.lines = []
        self._w = 0
        self._h = 0
        self.hide()

    def add_line(self, text: str, color: QColor):
        self.lines.append((text, color))
        fm = QFontMetrics(self.font)

        # marker = 8px + spacing 6px
        w = fm.horizontalAdvance(text) + 8 + 6
        h = fm.height()

        self._w = max(self._w, w)
        self._h += h

    def boundingRect(self) -> QRectF:
        return QRectF(0, 0,
                      self._w + self.padding * 2,
                      self._h + self.padding * 2)

    def paint(self, painter: QPainter, option, widget=None):
        if not self.lines:
            return

        painter.setRenderHint(QPainter.Antialiasing)
        painter.setFont(self.font)
        fm = QFontMetrics(self.font)

        # Background
        rect = self.boundingRect()
        painter.setBrush(QBrush(self.bg_color))
        painter.setPen(QPen(self.border_color, 1))
        painter.drawRoundedRect(rect, self.border_radius, self.border_radius)

        # Content
        y = self.padding

        for text, color in self.lines:
            # --- marker ---
            painter.setBrush(color)
            painter.setPen(Qt.NoPen)
            painter.drawEllipse(self.padding,
                                y + fm.height() / 2 - 4, 8, 8)

            # --- text ---
            painter.setPen(QPen(QColor("white")))
            painter.drawText(self.padding + 14,
                             y + fm.ascent(), text)

            y += fm.height()


# -----------------------------------------------------
# ChartView with hover + leave
# -----------------------------------------------------
class HoverChartView(QChartView):
    def __init__(self, chart, parent=None):
        super().__init__(chart, parent)
        self.owner: 'ChartSLine' = None
        self.setMouseTracking(True)

    def mouseMoveEvent(self, event):
        if self.owner:
            self.owner.handle_hover(event)
        super().mouseMoveEvent(event)

    def leaveEvent(self, event):
        if self.owner:
            self.owner.hide_hover()
        super().leaveEvent(event)


# -----------------------------------------------------
# ChartSLine
# -----------------------------------------------------
class ChartSLine(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        type: str = "line",
        series: object = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: object = None,
        key: str = None,
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self._series_input = series or []
        self._options = options or {}
        self._width = width
        self._height = height

        self._init_chart()

        theme = useTheme()
        theme.state.valueChanged.connect(self._apply_theme)
        self._apply_theme()

    # -----------------------------------------------------
    # INIT CHART
    # -----------------------------------------------------
    def _init_chart(self):
        if self._height:
            self.setFixedHeight(self._height)
        if self._width:
            self.setFixedWidth(self._width)

        # Build series
        self._spline_series_list = []
        for item in self._series_input:
            s = QSplineSeries()
            s.setName(item.get("name", ""))

            for idx, y in enumerate(item.get("data", [])):
                s.append(float(idx), float(y))

            self._spline_series_list.append(s)

        # Chart
        self.chart = QChart()
        self.chart.legend().setVisible(True)
        self.chart.legend().setAlignment(Qt.AlignTop)

        for s in self._spline_series_list:
            self.chart.addSeries(s)

        # Axes
        self.axis_y = QValueAxis()
        self.axis_y.setLabelFormat("%d")

        categories = self._options.get("xaxis", {}).get("categories", None)
        if categories:
            self.axis_x = QBarCategoryAxis()
            self.axis_x.append(categories)
            self.axis_y.setRange(0, max(max(item["data"]) for item in self._series_input))
        else:
            self.axis_x = QValueAxis()

        self.chart.addAxis(self.axis_x, Qt.AlignBottom)
        self.chart.addAxis(self.axis_y, Qt.AlignLeft)

        for s in self._spline_series_list:
            s.attachAxis(self.axis_x)
            s.attachAxis(self.axis_y)

        # Hover markers
        self.hover_markers = []
        for s in self._spline_series_list:
            mk = QScatterSeries()
            mk.setMarkerShape(QScatterSeries.MarkerShapeCircle)
            mk.setMarkerSize(12)
            mk.setColor(QColor(s.pen().color()))
            mk.setBorderColor(QColor("#ffffff"))
            mk.hide()

            self.chart.addSeries(mk)
            mk.attachAxis(self.axis_x)
            mk.attachAxis(self.axis_y)
            self.hover_markers.append(mk)

        # Vertical line
        self.hover_line = QLineSeries()
        self.hover_line.setPen(QPen(QColor("#8888ff"), 1, Qt.DashLine))
        self.hover_line.hide()
        self.chart.addSeries(self.hover_line)
        self.hover_line.attachAxis(self.axis_x)
        self.hover_line.attachAxis(self.axis_y)

        # Tooltip
        self.tooltip = TooltipItem(self.chart)
        self.tooltip.hide()

        # ChartView
        self.chart_view = HoverChartView(self.chart)
        self.chart_view.owner = self
        self.chart_view.setRenderHint(QPainter.Antialiasing)
        self.chart_view.scene().addItem(self.tooltip)

        self.layout().addWidget(self.chart_view)

    # -----------------------------------------------------
    # HIDE hover components
    # -----------------------------------------------------
    def hide_hover(self):
        self.tooltip.clear()
        self.hover_line.hide()
        for mk in self.hover_markers:
            mk.hide()

    # -----------------------------------------------------
    # Hover Handler
    # -----------------------------------------------------
    def handle_hover(self, event):
        pos = event.pos()

        if not self._spline_series_list:
            return

        s0 = self._spline_series_list[0]
        chart_pos = self.chart.mapToValue(pos, s0)
        x = chart_pos.x()

        nearest_x = round(x)
        nearest_x = max(0, min(nearest_x, s0.count() - 1))

        # collect points
        pts = [s.at(nearest_x) for s in self._spline_series_list]

        # markers update
        for i, mk in enumerate(self.hover_markers):
            mk.clear()
            mk.append(pts[i])
            mk.show()

        # vertical line update
        ymin, ymax = self.axis_y.min(), self.axis_y.max()
        self.hover_line.clear()
        self.hover_line.append(nearest_x, ymin)
        self.hover_line.append(nearest_x, ymax)
        self.hover_line.show()

        # tooltip content
        self.tooltip.clear()

        categories = self._options.get("xaxis", {}).get("categories")
        title = categories[nearest_x] if categories else f"Index {nearest_x}"
        self.tooltip.add_line(title, QColor("#cccccc"))

        for i, s in enumerate(self._spline_series_list):
            self.tooltip.add_line(
                f"{s.name()}: {pts[i].y():.2f}",
                s.pen().color()
            )

        self.tooltip.prepareGeometryChange()

        # tooltip position
        plot_area = self.chart.plotArea()
        ref = self.chart.mapToPosition(QPointF(nearest_x, ymax), s0)

        rect = self.tooltip.boundingRect()
        w = rect.width()

        if ref.x() < plot_area.center().x():
            tx = ref.x() + 15
        else:
            tx = ref.x() - w - 25

        ty = ref.y() + 10

        self.tooltip.setPos(tx, ty)
        self.tooltip.show()

    # -----------------------------------------------------
    # Apply Theme
    # -----------------------------------------------------
    def _apply_theme(self):
        theme = useTheme()

        text_color = QColor(theme.palette.text.secondary)
        primary_color = QColor(theme.palette.primary.main)

        self.chart.setTitleBrush(text_color)

        font = QFont()
        font.setPointSize(8)
        font.setWeight(QFont.Weight(600))

        pen = QPen(Qt.DashLine)
        pen.setWidthF(0.2)
        pen.setColor(text_color)

        self.axis_x.setGridLineVisible(False)
        self.axis_x.setLabelsBrush(text_color)
        self.axis_x.setLinePen(pen)
        self.axis_x.setLabelsFont(font)

        self.axis_y.setLinePen(Qt.NoPen)
        self.axis_y.setGridLinePen(pen)
        self.axis_y.setLabelsFont(font)

        legend = self.chart.legend()
        legend.setLabelBrush(text_color)

        # series pen
        for idx, s in enumerate(self._spline_series_list):
            p = QPen(primary_color if idx == 0 else QColor(theme.palette.info.main))
            p.setWidth(3)
            s.setPen(p)



import asyncio
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QFrame, QWidget, QScrollArea, QSizePolicy, QApplication, QLabel
from PySide6.QtGui import QGuiApplication
from PySide6.QtCore import Qt, QRunnable, QTimer, QThreadPool
import uuid

from qtmui.hooks import State

class WidgetSetter(QRunnable):
    def __init__(self, cls, children):
        super().__init__()
        self.cls= cls
        self.children=children

    async def run(self):
        for index, widget in enumerate(self.children):
            await asyncio.sleep(0.1)
            # print(f"add widget {index} before: ", self.cls.widgetContents.height())
            self.cls.vlayout.addWidget(widget)
            self.cls.widgetContents.adjustSize()
            self.cls.scroll_area.widget().adjustSize()
            self.cls.adjustSize()
            self.cls.scroll_area.setFixedHeight(self.cls.widgetContents.height())
            self.cls.setFixedHeight(self.cls.widgetContents.height())
            self.cls.parent().setFixedHeight(self.cls.widgetContents.height())
            self.cls.widgetContents.parent().setFixedHeight(self.cls.widgetContents.height())
            self.cls.parent().parent().setFixedHeight(self.cls.widgetContents.height())
            self.cls.adjustSize()
            self.cls.parent().adjustSize()
            self.cls.parent().updateGeometry()
            self.cls.updateGeometry()
            self.cls.scroll_area.adjustSize()
            self.cls.scroll_area.updateGeometry()
            print('self.cls', self.cls.widgetContents.parent().height(), self.cls.scroll_area.height(), self.cls.parent().height(), self.cls.height(), self.cls.scroll_area.widget().height(), self.cls.parent().parent().height())
            # print(f"add widget {index} after: ", self.cls.widgetContents.height())

    
class MenuScrollArea(QFrame):

    def __init__(self,  
                sx: dict = None,
                maxWidth: int = None,
                defaultWidth: int = None,
                hightLight: bool = None,
                backgroundColor: str = None,
                children: list = None,
                fullWidth: bool = False,
                spacing=6,
                border: str = '',
                borderForNav: bool = None,
        ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(spacing)
        self._border = border
        
        # self._thread_pool = QThreadPool.globalInstance()  # Sử dụng thread pool toàn cục
        
        self.scroll_area = QScrollArea(self)
        self.layout().addWidget(self.scroll_area)

        self.scroll_area.setWidgetResizable(True)
        self.widgetContents = QWidget(self.scroll_area)

        self.vlayout = QVBoxLayout(self.widgetContents)
        self.vlayout.setContentsMargins(0,0,0,0)
        self.vlayout.setSpacing(3)
        
        self.scroll_area.setWidget(self.widgetContents)
            

        if backgroundColor is not None:
            if isinstance(backgroundColor, State):
                backgroundColor.valueChanged.connect(self._set_background)
                self._set_background(backgroundColor.value)
            else:
                self._set_background(background=backgroundColor)

        if maxWidth is not None:
            self.setMaximumWidth(maxWidth)

        if defaultWidth is not None:
            self.resize(defaultWidth, self.height())

        if isinstance(children, list) and len(children) > 0:
            for item in children:
                if isinstance(item, QWidget):
                    self.vlayout.addWidget(item)
                    # self._add_child_fast(item)
                    # print('itemmmmmmmmmmmmm', item)

        # self.worker = WidgetSetter(self, children)
        # self._thread_pool.start(QTimer.singleShot(0, lambda: asyncio.ensure_future(self.worker.run())))


        if fullWidth:
            self.setFullWidth()

        # self.installEventFilter(self)
        self._set_stylesheet()
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)

    def _add_child_fast(self, widget):
        self._thread_pool.start(QTimer.singleShot(0, lambda widget=widget: asyncio.ensure_future(self.worker.run())))  # Chạy setIndexWidget trong thread riêng

    def _set_stylesheet(self):
        # print('_set_background__________container', background)
        # print('_set_background___________border', self._border)
        self.setStyleSheet(f"""

            QWidget {{
                background-color: transparent;
            }}
            QScrollBar:horizontal {{
                border: none;
                background: rgb(249, 249, 249);
                height: 4px;
                margin: 0px 21px 0 21px;
                border-radius: 0px;
            }}

            QScrollBar::handle:horizontal {{
                background: rgb(227, 227, 227);
                min-width: 25px;
                border-radius: 0px
            }}

            QScrollBar::add-line:horizontal {{
                border: none;
                background: rgb(227, 227, 227);
                width: 20px;
                border-top-right-radius: 0px;
                border-bottom-right-radius: 0px;
                subcontrol-position: right;
                subcontrol-origin: margin;
            }}

            QScrollBar::sub-line:horizontal {{
                border: none;
                background: transparent;
                width: 20px;
                border-top-left-radius: 0px;
                border-bottom-left-radius: 0px;
                subcontrol-position: left;
                subcontrol-origin: margin;
            }}

            QScrollBar::up-arrow:horizontal,
            QScrollBar::down-arrow:horizontal {{
                background: none;
            }}

            QScrollBar::add-page:horizontal,
            QScrollBar::sub-page:horizontal {{
                background: none;
            }}

            QScrollBar:vertical {{
                border: none;
                background: rgb(249, 249, 249);
                width: 4px;
                margin: 21px 0 21px 0;
                border-radius: 0px;
            }}

            QScrollBar::handle:vertical {{
                background: rgb(227, 227, 227);
                min-height: 25px;
                border-radius: 0px;
            }}


            QScrollBar::add-line:vertical {{
                border: none;
                background: transparent;
                height: 20px;
                border-bottom-left-radius: 0px; 
                border-bottom-right-radius: 0px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
            }}

            QScrollBar::sub-line:vertical {{
                border: none;
                background: transparent;
                height: 20px;
                border-top-left-radius: 0px;
                border-top-right-radius: 0px;
                subcontrol-position: top;
                subcontrol-origin: margin;
            }}

            QScrollBar::up-arrow:vertical,
            QScrollBar::down-arrow:vertical {{
                background: none;
            }}

            QScrollBar::add-page:vertical,
            QScrollBar::sub-page:vertical {{
                background: none;
            }}
        """)

    def _set_background(self, background):
        self.setStyleSheet(self.styleSheet() + f"""
            #{self.objectName()} {{
                background-color: {background};
                {self._border}
            }}
        """)


    def setFullWidth(self):
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.vlayout.setSpacing(0)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

    # def eventFilter(self, obj, event):
    #     if event.type() == QEvent.Resize:
    #         self.adjustWidth()
    #     return super().eventFilter(obj, event)

    def adjustWidth(self):
        screen_size = QGuiApplication.primaryScreen().size()
        screen_width = screen_size.width()
        if screen_width < 576:
            self.setMaximumWidth(screen_width)
        elif screen_width < 768:
            self.setMaximumWidth(540)
        elif screen_width < 992:
            self.setMaximumWidth(720)
        elif screen_width < 1200:
            self.setMaximumWidth(960)
        elif screen_width < 1400:
            self.setMaximumWidth(1140)
        else:
            self.setMaximumWidth(1320)


from __future__ import annotations
from typing import Optional, Union
from math import pi, cos, sin
from PySide6.QtCore import Qt, QPointF, QRectF
from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel, QHBoxLayout
from PySide6.QtGui import QGradient, QLinearGradient, QPainter, QColor, QBrush, QFont, QPen, QPainterPath


from qtmui.material.styles import useTheme

class ChartRadialBar1(QWidget):
    def __init__(
        self,
        series: list = None,
        total: int = None,
        options: dict = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)
        self._series = series or []
        self._total = total
        self._options = options
        self._width = int(width) if width else 200
        self._height = int(height) if height else 200
        self.percentage = 0
        self.theme = useTheme()
        
        self.setMinimumSize(self._width, self._height)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self._init_widget()

        self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _init_widget(self):
        if self._width:
            self.setFixedWidth(self._width)

        if self._height:
            self.setFixedHeight(self._height)
        self._add_center_label()

    def upd(self, data: list):
         self._series = data
         self.update()


    def paintEvent(self, e):
        p = QPainter(self)
        p.setRenderHint(QPainter.Antialiasing)
        p.fillRect(self.rect(), self.theme.palette.background.paper)

        # Calculate the drawing rectangle
        rect = self.rect().adjusted(8, 8, -8, -8)  # reduce for pen width
        side = min(rect.width(), rect.height())
        x = rect.x() + (rect.width() - side) / 2
        y = rect.y() + (rect.height() - side) / 2
        draw_rect = QRectF(x, y, side, side)

        start_angle = 270 - 180  # Start at top (12 o'clock -> 270 - 90 degrees)
        pen_width = 8
        gap = 6  # Gap between arcs
        num_arcs = len(self._series)
        for i, (_, value, color) in enumerate(self._series):
             
                end_angle = -(value / self._total) * 360 if self._total > 0 else 0  # Negative for clockwise
                path = QPainterPath()
                center = draw_rect.center()
                
                #Calculate the radius for the current arc
                current_radius = draw_rect.width()/2 - ((num_arcs - i -1) * (pen_width + gap) + pen_width/2)
                
                #Calculate the rectangle containing the current arc.
                arc_rect = QRectF(
                   center.x() - current_radius,
                   center.y() - current_radius,
                   current_radius * 2,
                   current_radius * 2
                )
                path.moveTo(center.x(), center.y())


                # Create an arc from center to edge.
                path.arcMoveTo(arc_rect, start_angle)
                path.arcTo(arc_rect, start_angle, end_angle)

                pen = QPen()
                pen.setCapStyle(Qt.FlatCap)
                pen.setColor(QColor(getattr(self.theme.palette, color).main))
                pen.setWidth(pen_width)
                p.strokePath(path, pen)




    def _add_center_label(self):
        if self._total is None:
            return

        # Create the label for 'Total'
        total_label = QLabel("Total", self)
        total_label.setAlignment(Qt.AlignCenter)
        total_label.setStyleSheet(f"color: {self.theme.palette.text.secondary};")

        # Create the label for the total value
        self.total_value_label = QLabel(str(self._total), self)
        self.total_value_label.setAlignment(Qt.AlignCenter)
        self.total_value_label.setStyleSheet(f"color: {self.theme.palette.text.primary}; font-size: 14px; font-weight: bold;")

        # Create a layout to place both labels in the center
        center_layout = QVBoxLayout()
        center_layout.addWidget(total_label)
        center_layout.addWidget(self.total_value_label)
        center_layout.setContentsMargins(0, 0, 0, 0)
        center_layout.setAlignment(Qt.AlignCenter | Qt.AlignVCenter) # Add AlignVCenter

        # Create a wrapper widget
        center_widget = QWidget(self)
        center_widget.setLayout(center_layout)

        # Add the widget to the main layout
        self.layout().addWidget(center_widget)
        self.layout().setAlignment(Qt.AlignCenter)  # Center the wrapper widget

        #  The text should now be perfectly centered

        # Adjust the size of label when the chart size changes.
        self.resizeEvent = lambda event: self._update_center_label_position(center_widget)

    def _update_center_label_position(self, center_widget):
       
        # Calculate the drawing rectangle
        rect = self.rect().adjusted(8, 8, -8, -8)  # reduce for pen width
        side = min(rect.width(), rect.height())
        x = rect.x() + (rect.width() - side) / 2
        y = rect.y() + (rect.height() - side) / 2
        draw_rect = QRectF(x, y, side, side)
        
        center_widget.setGeometry(QRectF(
            draw_rect.x(), 
            draw_rect.y(),
            draw_rect.width(),
            draw_rect.height()
        ).toRect())


    def _set_stylesheet(self):
        self.theme = useTheme()
        self.update()
        if hasattr(self, "total_value_label"):
            self.total_value_label.setStyleSheet(f"color: {self.theme.palette.text.primary}; font-size: 14px; font-weight: bold;")



from typing import TYPE_CHECKING, Optional

from ....system.color_manipulator import alpha

from .properties_name import *


if TYPE_CHECKING:
    from ..theme_reducer import ThemeState
    from qtmui.material.styles.create_theme.create_palette import Palette


def button(_theme):
    # Khởi tạo các thông tin cần thiết
    theme: ThemeState = _theme
    theme_palette: Palette = theme.palette

    COLORS = ['primary', 'secondary', 'info', 'success', 'warning', 'error']
    lightMode = theme.palette.mode == 'light'
    
    
    return {
        'MuiButton': {
            'styles': {
                'root': lambda ownerState: {
                    "colorStyle": {
                        **{
                            _color: {
                                "props": {
                                    "containedVariant": {
                                        color: theme.palette.common.white if lightMode else theme.palette.grey._800,
                                        backgroundColor: getattr(theme.palette, _color).main,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: getattr(theme_palette, _color).dark if lightMode else getattr(theme_palette, _color).light,
                                            },
                                            "checked": {
                                                backgroundColor: getattr(theme_palette, _color).dark if lightMode else getattr(theme_palette, _color).light,
                                            },
                                        }
                                    },
                                    "outlinedVariant": {
                                        border: f"1px solid {alpha(getattr(theme.palette, _color).main, 0.32)}",
                                        backgroundColor: "transparent",
                                        color: getattr(theme.palette, _color).main,
                                        "slots": {
                                            "hover": {
                                                borderColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                            },
                                            "checked": {
                                                borderColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                            },
                                        }
                                    },
                                    "textVariant": {
                                        border: f"1px solid transparent",
                                        backgroundColor: "transparent",
                                        color: getattr(theme.palette, _color).main,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                            },
                                            "checked": {
                                                backgroundColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                            },
                                        }
                                    },
                                    "softVariant": {
                                        border: f"1px solid transparent",
                                        backgroundColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                        color: getattr(theme_palette, _color).dark if lightMode else getattr(theme_palette, _color).light,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: alpha(getattr(theme.palette, _color).main, 0.32),
                                            },
                                            "checked": {
                                                backgroundColor: alpha(getattr(theme.palette, _color).main, 0.32),
                                            },
                                        }
                                    },
                                    "disabledVariant": {
                                        border: f"1px solid transparent",
                                        backgroundColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                        color: getattr(theme_palette, _color).dark if lightMode else getattr(theme_palette, _color).light,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: alpha(getattr(theme.palette, _color).main, 0.32),
                                            },
                                            "checked": {
                                                backgroundColor: alpha(getattr(theme.palette, _color).main, 0.32),
                                            },
                                        }
                                    },
                                },
                            }
                            for _color in COLORS
                        },
                        **{
                            "default": {
                                "props": {
                                    "containedVariant": {
                                        border: "1px solid transparent",
                                        backgroundColor: theme.palette.common.white if not lightMode else theme.palette.grey._800,
                                        color: theme.palette.common.white if lightMode else theme.palette.grey._800,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: theme.palette.common.white if not lightMode else theme.palette.grey._800,
                                            },
                                            "checked": {
                                                backgroundColor: theme.palette.common.white if not lightMode else theme.palette.grey._800,
                                            },
                                        }
                                    },
                                    "outlinedVariant": {
                                        border: "1px solid transparent",
                                        borderColor: alpha(theme_palette.grey._500, 0.32),
                                        color: theme.palette.common.white if not lightMode else theme.palette.grey._800,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: theme_palette.action.hover,
                                            },
                                            "checked": {
                                                backgroundColor: theme_palette.action.hover,
                                            },
                                        }
                                    },
                                    "textVariant": {
                                        border: "1px solid transparent",
                                        borderColor: "transparent",
                                        color: theme.palette.grey._500,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: theme_palette.action.hover,
                                            },
                                            "checked": {
                                                backgroundColor: theme_palette.action.hover,
                                            },
                                        }
                                    },
                                    "softVariant": {
                                        border: "1px solid transparent",
                                        backgroundColor: alpha(theme_palette.grey._500, 0.08),
                                        color: theme.palette.common.white if not lightMode else theme.palette.grey._800,
                                        "slots": {
                                            "hover": {
                                                backgroundColor: alpha(theme_palette.grey._500, 0.24),
                                            },
                                            "checked": {
                                                backgroundColor: alpha(theme_palette.grey._500, 0.24),
                                            },
                                        }
                                    },
                                },
                            },
                        },
                    },

                    "size": {
                        lineHeight: theme.typography.button.lineHeight,
                        fontWeight: theme.typography.button.fontWeight,
                        border: "1px solid transparent",
                        borderRadius: "4px" if ownerState.get("size") == "small" else "6px" if ownerState.get("size") == "medium" else "8px",
                        height: "30px" if ownerState.get("size") == "small" else "36px" if ownerState.get("size") == "medium" else "48px",
                        fontSize: 13 if ownerState.get("size") == "small" else 14 if ownerState.get("size") == "medium" else 15,
                        paddingLeft: "8px" if ownerState.get("size") == "small" else "12px" if ownerState.get("size") == "medium" else "16px",
                        paddingRight: "8px" if ownerState.get("size") == "small" else "12px" if ownerState.get("size") == "medium" else "16px",
                        'textVariant': {
                            paddingLeft: "4px" if ownerState.get("size") == "small" else "8px" if ownerState.get("size") == "medium" else "14px", # mui 4
                            paddingRight: "4px" if ownerState.get("size") == "small" else "8px" if ownerState.get("size") == "medium" else "14px", # mui 4
                        }
                    },
                },
                'icon': {
                    "size": {
                        'small': {
                            width: 16,
                            height: 16,
                        },
                        'medium': {
                            width: 20,
                            height: 20,
                        },
                        'large': {
                            width: 24,
                            height: 24,
                        }
                    }
                },
            },
        },


    }



import asyncio
import threading
from typing import Callable

from PySide6.QtCore import Qt, QModelIndex, Signal, QTimer
from PySide6.QtWidgets import QHeaderView, QAbstractItemView, QTableView, QProxyStyle, QStyleOption

from .frozen_column import FrozenColumn

from .data_grid_model import DataGridModel
from ..widget_base import PyWidgetBase

from qtmui.material.styles import useTheme

class DataGrid(QTableView, PyWidgetBase):

  setIndexWidgetSignal = Signal(QModelIndex)

  class DropmarkerStyle(QProxyStyle):
    def drawPrimitive(self, element, option, painter, widget=None):
      """Draw a line across the entire row rather than just the column we're hovering over.
      This may not always work depending on global style - for instance I think it won't
      work on OSX."""
      if str(
        element) == "PrimitiveElement.PE_IndicatorItemViewItemDrop" and not option.rect.isNull():  # element == self.PE_IndicatorItemViewItemDrop and
        option_new = QStyleOption(option)
        option_new.rect.setLeft(0)
        if widget:
          option_new.rect.setRight(widget.width())
        option = option_new
      super().drawPrimitive(element, option, painter, widget)

  def __init__(
      self,

      columns,
      rows,
      checkboxSelection=False,
      disableRowSelectionOnClick=False,

      context=None,
      id: str = None,
      isDense: bool = None,
      isManyRows: bool = None,
      fullWidth: bool = True,
      tableHead: list = None,
      singleSelection: bool = False,
      stretchLastSection: bool = False,
      model=None,
      selectionChanged: Callable = None,
      itemDelegates=None,
      backgroundColor: str = "#ffffff",
      isFrozenLastColumn=None
      ):
    super(DataGrid, self).__init__()
    if id is not None:
      self.setObjectName(id)


    self._columns = columns
    self._rows = rows
    self._checkboxSelection = checkboxSelection
    self._disableRowSelectionOnClick = disableRowSelectionOnClick
    
    self._background_color = "transparent"
    self._isDense = isDense
    self._isManyRows = isManyRows
    self._fullWidth = fullWidth
    self._tableHead = tableHead
    # self._model = model
    self._stretchLastSection = stretchLastSection
    self._isFrozenLastColumn = isFrozenLastColumn
    self._itemDelegates = itemDelegates
    self._selectionChanged = selectionChanged
    self._singleSelection = singleSelection

    self._frozenColumn = None

    self._is_filter_mode = False
    self._column_count = len(self._columns)

    self.last_drop_row = None
    self.lastCol = len(self._columns) - 1

    # if self._selectionChanged is not None:
    #   # self.pressed.connect(self._selectionChanged)
    #   self.selectionModel().selectionChanged.connect(self._selectionChanged)

    self._model = DataGridModel(self, self._columns, self._rows)


    self.setAttribute(Qt.WA_Hover, True)

    self.setupUi()

    self.setIndexWidgetSignal.connect(self._set_index_widget)


  def _thread_set_index_widget(self):
    for i in range(len(self._columns)):
      for j in range(len(self._rows)):
        self.setIndexWidgetSignal.emit(self._model.index(j, i))

  def _set_index_widget(self, index):
    renderCell = self._columns[index.column()].get("renderCell")
    if renderCell:
      options = self._rows[index.row()]
      cell = renderCell(options)
      self.setIndexWidget(index, cell)


  def setupUi(self):
    self.setDragEnabled(True)
    self.setAcceptDrops(True)
    self.setDragDropOverwriteMode(False)
    # self.setStyle(self.DropmarkerStyle())

    if self._isFrozenLastColumn == True:
      self._frozenColumn = FrozenColumn(parent=self, lastCol=self.lastCol)

    self.update_model(self._model)

    self._horizontalHeader = self.horizontalHeader()
    self._horizontalHeader.setMinimumWidth(200)

    self.horizontalHeader().sectionResized.connect(self.updateSectionWidth)
    self.verticalHeader().sectionResized.connect(self.updateSectionHeight)

    if self._stretchLastSection == True:
      self.horizontalHeader().setStretchLastSection(True)

    if self._singleSelection:
      self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
    else:
      self.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
      self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)

    if self._isFrozenLastColumn == True:
      self.viewport().stackUnder(self._frozenColumn)
    self.viewport().setContentsMargins(0, 0, 0, 0)

    self.header = self.horizontalHeader()
    self.header.setDefaultAlignment(Qt.AlignLeft)
    
    self.verticalHeader().hide()

    self.setViewportMargins(0, 0, 0, 0)
    self.setSortingEnabled(True)
    self.setDragEnabled(True)

    self.setWordWrap(False)

    self.verticalHeader().setDefaultSectionSize(40)

    self.theme = useTheme()
    self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
    self.destroyed.connect(self._on_destroyed)
    self.slot_set_stylesheet()

  def slot_set_stylesheet(self, value=None):
      self._set_stylesheet()

  def _set_stylesheet(self, component_styled=None):
    self.theme = useTheme()

    ownerState = {}

    if not component_styled:
        component_styled = self.theme.components

    self.setStyleSheet(f"""
        QWidget {{
                background-color: transparent;
        }}
        QTableView {{
            background-color: transparent;
            padding: 0px;
            border: none;
            gridline-color: transparent;
            color: {self.theme.palette.text.secondary};
        }}

        QTableView::item {{
            padding-left: 0px;
            padding-right: 5px;
        }}

        QTableView::item:selected {{
            background-color: {self.theme.palette.action.selected};
            color: {self.theme.palette.text.secondary};
        }}

        QTableView::section {{
            background-color: transparent;
            max-width: 30px;
            text-align: left;

        }}

        QTableView::horizontalHeader {{
            background-color: red;
            color: {self.theme.palette.text.primary};
            font-size: {self.theme.typography.button.fontSize};
            font-weight: {self.theme.typography.button.fontWeight};
            line-height: {self.theme.typography.button.lineHeight};
        }}

        QTableView::section:horizontal {{
            background-color: transparent;
            padding: 0px;
            border: 1px solid red !important;
            border-bottom: 1px solid red !important;
        }}


        QTableView::section:vertical {{
            border: 1px solid red;
        }}

        QTableView .QScrollBar:horizontal {{
            border: none;
            background: transparent;
            min-height: 8px;
            border-radius: 0px;
            max-width: 79em;
        }}

        QTableView .QHeaderView::section {{
            background-color: transparent!important;
        }}

      """)

  def update_model(self, model=None):
    # print('update_model______________', self._model._data)
    if model == None:
      model = self._model

    self.setModel(model)
    # self.update(model)

    # self.setItemDelegateForColumn(5, ButtonDelegate(self))

    # if isinstance(self._itemDelegates, list):
    #   for item in self._itemDelegates:
    #     item.parent = self
    #     self.setItemDelegateForColumn(2, item)

    if self._isFrozenLastColumn == True:
      self._frozenColumn.setModel(model)
      self._frozenColumn.setupUi()
      self.updateFrozenTableGeometry()
      self._frozenColumn.setFocusPolicy(Qt.NoFocus)
      self._frozenColumn.configTable(self.selectionModel())
      self._itemDelegates[-1].parent = self._frozenColumn
      self._frozenColumn.setItemDelegateForColumn(self.lastCol, self._itemDelegates[-1])
      if not self.isVisible():
        self.show()

    if self._selectionChanged is not None:
      # self.pressed.connect(self._selectionChanged)
      self.selectionModel().selectionChanged.connect(self._selectionChanged)

    self.configUiTableView()

  def configUiTableView(self):
    self.verticalHeader().hide()

    self.setViewportMargins(0, 0, 0, 0)
    
    self._horizontalHeader = self.horizontalHeader()
    self.horizontalHeader().setSectionResizeMode(5, QHeaderView.Stretch)

    # for index, item in enumerate(self._tableHead):
    #   if item.get('resizeMode') == "ResizeToContents":
    #     self._horizontalHeader.setSectionResizeMode(index, QHeaderView.ResizeToContents)
    #   if item.get('resizeMode') == "Fixed":
    #     self._horizontalHeader.setSectionResizeMode(index, QHeaderView.Fixed)
    #     self.setColumnWidth(index, item.get('width'))
    #   if item.get('resizeMode') == "Stretch":
    #     self._horizontalHeader.setSectionResizeMode(index, QHeaderView.Stretch)
    #   if item.get('columnHidden') == True:
    #     self.setColumnHidden(index, True)

    # sorted_dict = {
    #     "align": 'right',
    #     "description": 'This column has a value getter and is not sortable.',
    #     "disableColumnMenu": True,
    #     "editable": True,
    #     "field": 'age',
    #     "flex": 1,
    #     "headerAlign": 'center',
    #     "headerName": 'Age',
    #     "renderCell": lambda: print('renderCell'),
    #     "sortable": False,
    #     "type": 'number',
    #     "valueGetter": lambda params: f'{params.row.firstName | ""} {params.row.lastName | ""}',
    #     "width": 120,
    # }

    for index, item in enumerate(self._columns):
      if item.get('align'):
        pass
      if item.get('description'):
        pass
      if item.get('disableColumnMenu'):
        pass
      if item.get('editable'):
        pass
      if item.get('field'):
        pass
      if item.get('flex'):
        pass
      if item.get('headerAlign'):
        pass
      if item.get('headerName'):
        pass
      if item.get('renderCell'):
        # delegate = item.get('renderCell')()
        # delegate = item.get('renderCell')
        # delegate.parent = self
        # print('delegate______________', index, delegate)
        # self.setItemDelegateForColumn(index, delegate)
        pass
      if item.get('sortable'):
        pass
      if item.get('type'):
        pass
      if item.get('valueGetter'):
        pass
      if item.get('width'):
        pass


    



  def updateSectionWidth(self, logicalIndex, oldSize, newSize):
    if self._isFrozenLastColumn is not None and logicalIndex == self.lastCol:
      self._frozenColumn.setColumnWidth(self.lastCol, newSize)
    self.updateFrozenTableGeometry()

  def updateSectionHeight(self, logicalIndex, oldSize, newSize):
    if  self._frozenColumn is not None:
      self._frozenColumn.setRowHeight(logicalIndex, newSize)

  def resizeEvent(self, event):
    super(DataGrid, self).resizeEvent(event)
    self.updateFrozenTableGeometry()

  def moveCursor(self, cursorAction, modifiers):
    current = super(DataGrid, self).moveCursor(cursorAction, modifiers)
    if (cursorAction == QAbstractItemView.MoveLeft and current.column() < self.lastCol and
      self.visualRect(current).topLeft().x() < (self._frozenColumn.columnWidth(self.lastCol))):
      newValue = (self.horizontalScrollBar().value() +
                  self.visualRect(current).topLeft().x() - self._frozenColumn.columnWidth(self.lastCol))
      self.horizontalScrollBar().setValue(newValue)
    return current

  def scrollTo(self, index, hint):
    if index.column() < self.lastCol:
      super(DataGrid, self).scrollTo(index, hint)

  def updateFrozenTableGeometry(self):
    x_position = self.verticalHeader().width() + self.frameWidth()
    for col in range(0, self.lastCol):
      x_position += self.columnWidth(col)
    x_viewPort = self.verticalHeader().width() + self.viewport().width() - self.columnWidth(
      self.lastCol) + self.frameWidth()
    if self._isFrozenLastColumn is not None:
      self._frozenColumn.setGeometry(x_position if x_position < x_viewPort else x_viewPort,
                                      self.frameWidth(), self.columnWidth(self.lastCol),
                                      self.viewport().height() + self.horizontalHeader().height())


  def cellButtonClicked(self, data):
    print('data_________', data)
    arr_info = data.split('_')
    btn_name = arr_info[0]
    index_row = int(arr_info[1])
    id_profile = self.model().index(index_row, 1).data()
    profile_name = self._main_window.tblProfile._model._data[index_row][3]
    self.profile_id_selected = id_profile


  def dropEvent(self, event):
    sender = event.source()
    super().dropEvent(event)
    dropRow = self.last_drop_row
    destination = self.objectName()
    to_index = self.indexAt(event.pos()).row()

    selectedRows = sender.getselectedRowsFast()

    arr_id_profile = []

    model = sender.model()
    for srow in selectedRows:
      id = model.index(srow, 1).data()
      arr_id_profile.append(int(id))

    event.accept()


  def getselectedRowsFast(self):
    selectedRows = []
    # for item in self.selectedItems():
    for item in self.selectedIndexes():
      if item.row() not in selectedRows:
        selectedRows.append(item.row())
    selectedRows.sort()
    return selectedRows






import asyncio
from typing import Any, Optional, Callable, Union, List, Dict
import uuid
from PySide6.QtWidgets import QFrame, QHBoxLayout, QLabel, QVBoxLayout, QToolButton, QWidget, QSizePolicy, QGraphicsDropShadowEffect
from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QColor
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.styles.create_theme.create_palette import PaletteColor
from qtmui.utils.translator import getTranslatedText
from ..system.color_manipulator import alpha, lighten_hex
from ...common.icon import FluentIconBase
from ..widget_base import PyWidgetBase
from ..py_tool_button.py_tool_button import PyToolButton
from qtmui.i18n.use_translation import translate, i18n
from qtmui.hooks import State
from ..utils.validate_params import _validate_param
from ...qtmui_assets import QTMUI_ASSETS

class Alert(QFrame, PyWidgetBase):
    """
    A component that displays an alert message with customizable severity, color, and actions.

    The `Alert` component is used to convey important information to the user, such as success,
    warning, error, or info messages. It supports various customization options including
    severity-based styling, icons, actions, and close buttons. It inherits properties from
    `QFrame` and `Paper` components, including `elevation`. Props of the Material-UI `Paper`
    component are also supported.

    Parameters
    ----------
    action : State, QWidget, list[QWidget], or None, optional
        The action to display at the end of the alert. Default is None.
        Can be a `State` object for dynamic updates.
    alertTitle : State or str, optional
        The title of the alert, displayed above the main message. Default is None.
        Can be a `State` object for dynamic updates.
    children : State, QWidget, list[QWidget], or None, optional
        The content of the component, typically text or widgets. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    closeText : State or str, optional
        The label for the close button. Default is "Close".
        Can be a `State` object for dynamic updates.
    color : State or str, optional
        The color of the component. If not provided, it is derived from the `severity` prop.
        Supports "error", "info", "success", "warning", or custom theme colors.
        Default is None. Can be a `State` object for dynamic updates.
    components : State or dict, optional
        The components used for each slot inside (e.g., CloseButton, CloseIcon).
        Default is {}. Deprecated: Use `slots` instead.
        Can be a `State` object for dynamic updates.
    componentsProps : State or dict, optional
        Extra props for the slot components. Default is {}.
        Deprecated: Use `slotProps` instead.
        Can be a `State` object for dynamic updates.
    elevation : State or int, optional
        The elevation level of the component, affecting the shadow depth (0-24).
        Default is 1. Can be a `State` object for dynamic updates.
    icon : State, PyToolButton, bool, or None, optional
        Override the icon displayed before the children. If False, no icon is displayed.
        Default is None. Can be a `State` object for dynamic updates.
    iconMapping : State or dict, optional
        Custom mapping of severity to icons. Default is None.
        Can be a `State` object for dynamic updates.
    key : State or str, optional
        A unique identifier for the alert. Default is None.
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, the alert takes up the full width of its container. Default is True.
        Can be a `State` object for dynamic updates.
    onClose : State or Callable, optional
        Callback fired when the component requests to be closed.
        Signature: function(event: Any) => None.
        Default is None. Can be a `State` object for dynamic updates.
    role : State or str, optional
        The ARIA role attribute of the element. Default is "alert".
        Can be a `State` object for dynamic updates.
    severity : State or str, optional
        The severity of the alert, defining its color and icon.
        Valid values: "error", "info", "success", "warning".
        Default is "success". Can be a `State` object for dynamic updates.
    size : State or str, optional
        The size of the alert (e.g., "small", "medium", "large"). Default is "medium".
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props used for each slot inside (e.g., action, closeButton, closeIcon, icon,
        message, root). Default is {}. Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components used for each slot inside (e.g., action, closeButton, closeIcon, icon,
        message, root). Default is {}. Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        The system prop for defining CSS overrides and additional styles.
        Default is None. Can be a `State` object for dynamic updates.
    text : State or str, optional
        The main message text of the alert.
        Default is "This is an info alert — check it out!".
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        The visual style of the alert. Valid values: "filled", "outlined", "standard".
        Default is "standard". Can be a `State` object for dynamic updates.

    Attributes
    ----------
    VALID_SEVERITIES : list[str]
        Valid values for the `severity` parameter: ["error", "info", "success", "warning"].
    VALID_VARIANTS : list[str]
        Valid values for the `variant` parameter: ["filled", "outlined", "standard"].

    Demos:
    - Alert: https://qtmui.com/material-ui/qtmui-alert/

    API Reference:
    - Alert API: https://qtmui.com/material-ui/api/alert/
    """

    VALID_SEVERITIES = ["error", "info", "success", "warning"]
    VALID_VARIANTS = ["filled", "outlined", "standard"]

    def __init__(
        self,
        action: Optional[Union[State, QWidget, List[QWidget]]] = None,
        alertTitle: Optional[Union[State, str]] = None,
        children: Optional[Union[State, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        closeText: Union[State, str] = "Close",
        color: Optional[Union[State, str]] = None,
        components: Optional[Union[State, Dict]] = None,
        componentsProps: Optional[Union[State, Dict]] = None,
        elevation: Optional[Union[State, int]] = 1,
        icon: Optional[Union[State, PyToolButton, bool]] = None,
        iconMapping: Optional[Union[State, Dict]] = None,
        key: Optional[Union[State, str]] = None,
        fullWidth: Union[State, bool] = True,
        onClose: Optional[Union[State, Callable[[Any], None]]] = None,
        role: Union[State, str] = "alert",
        severity: Union[State, str] = "success",
        size: Union[State, str] = "medium",
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        text: Optional[Union[str, State, Callable]] = "This is an info alert — check it out!",
        variant: Union[State, str] = "standard",
    ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))

        # Thiết lập các thuộc tính với dấu gạch dưới
        self._set_action(action)
        self._set_alertTitle(alertTitle)
        self._set_children(children)
        self._set_classes(classes)
        self._set_closeText(closeText)
        self._set_color(color)
        self._set_components(components)
        self._set_componentsProps(componentsProps)
        self._set_elevation(elevation)
        self._set_icon(icon)
        self._set_iconMapping(iconMapping)
        self._set_key(key)
        self._set_fullWidth(fullWidth)
        self._set_onClose(onClose)
        self._set_role(role)
        self._set_severity(severity)
        self._set_size(size)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)
        self._set_text(text)
        self._set_variant(variant)

        if not self._get_color():
            self._set_color(self._get_severity())

        self._btn_close = None
        self._lbl_alert_title = None
        self._lbl_alert_text = None
        self._frm_left = None
        self._frm_right = None
        self._frm_right_action = None
        self._init_ui()

    # @_validate_param(file_path="qtmui.material.alert", param_name="action", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_action(self, value):
        self._action = value

    def _get_action(self):
        return self._action.value if isinstance(self._action, State) else self._action

    @_validate_param(file_path="qtmui.material.alert", param_name="alertTitle", supported_signatures=Union[State, str, type(None)])
    def _set_alertTitle(self, value):
        self._alertTitle = value

    def _get_alertTitle(self):
        return self._alertTitle.value if isinstance(self._alertTitle, State) else self._alertTitle

    # @_validate_param(file_path="qtmui.material.alert", param_name="children", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.alert", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.alert", param_name="closeText", supported_signatures=Union[State, str])
    def _set_closeText(self, value):
        self._closeText = value

    def _get_closeText(self):
        return self._closeText.value if isinstance(self._closeText, State) else self._closeText

    @_validate_param(file_path="qtmui.material.alert", param_name="color", supported_signatures=Union[State, str, type(None)])
    def _set_color(self, value):
        self._color = value

    def _get_color(self):
        return self._color.value if isinstance(self._color, State) else self._color

    @_validate_param(file_path="qtmui.material.alert", param_name="components", supported_signatures=Union[State, Dict, type(None)])
    def _set_components(self, value):
        self._components = value or {}

    def _get_components(self):
        return self._components.value if isinstance(self._components, State) else self._components

    @_validate_param(file_path="qtmui.material.alert", param_name="componentsProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_componentsProps(self, value):
        self._componentsProps = value or {}

    def _get_componentsProps(self):
        return self._componentsProps.value if isinstance(self._componentsProps, State) else self._componentsProps

    # @_validate_param(file_path="qtmui.material.alert", param_name="elevation", supported_signatures=Union[State, int, type(None)], valid_values=range(0, 25))
    def _set_elevation(self, value):
        self._elevation = value

    def _get_elevation(self):
        return self._elevation.value if isinstance(self._elevation, State) else self._elevation

    @_validate_param(file_path="qtmui.material.alert", param_name="icon", supported_signatures=Union[State, PyToolButton, bool, type(None)])
    def _set_icon(self, value):
        self._icon = value

    def _get_icon(self):
        return self._icon.value if isinstance(self._icon, State) else self._icon

    @_validate_param(file_path="qtmui.material.alert", param_name="iconMapping", supported_signatures=Union[State, Dict, type(None)])
    def _set_iconMapping(self, value):
        self._iconMapping = value

    def _get_iconMapping(self):
        return self._iconMapping.value if isinstance(self._iconMapping, State) else self._iconMapping

    @_validate_param(file_path="qtmui.material.alert", param_name="key", supported_signatures=Union[State, str, type(None)])
    def _set_key(self, value):
        self._key = value

    def _get_key(self):
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(file_path="qtmui.material.alert", param_name="fullWidth", supported_signatures=Union[State, bool])
    def _set_fullWidth(self, value):
        self._fullWidth = value

    def _get_fullWidth(self):
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    @_validate_param(file_path="qtmui.material.alert", param_name="onClose", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClose(self, value):
        self._onClose = value

    def _get_onClose(self):
        return self._onClose.value if isinstance(self._onClose, State) else self._onClose

    @_validate_param(file_path="qtmui.material.alert", param_name="role", supported_signatures=Union[State, str])
    def _set_role(self, value):
        self._role = value

    def _get_role(self):
        return self._role.value if isinstance(self._role, State) else self._role

    @_validate_param(file_path="qtmui.material.alert", param_name="severity", supported_signatures=Union[State, str], valid_values=VALID_SEVERITIES)
    def _set_severity(self, value):
        self._severity = value

    def _get_severity(self):
        return self._severity.value if isinstance(self._severity, State) else self._severity

    @_validate_param(file_path="qtmui.material.alert", param_name="size", supported_signatures=Union[State, str])
    def _set_size(self, value):
        self._size = value

    def _get_size(self):
        return self._size.value if isinstance(self._size, State) else self._size

    @_validate_param(file_path="qtmui.material.alert", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value or {}

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.alert", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value or {}

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.alert", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.alert", param_name="text", supported_signatures=Union[State, str, Callable])
    def _set_text(self, value):
        self._text = value

    def _get_text(self):
        return self._text.value if isinstance(self._text, State) else self._text

    @_validate_param(file_path="qtmui.material.alert", param_name="variant", supported_signatures=Union[State, str], valid_values=VALID_VARIANTS)
    def _set_variant(self, value):
        self._variant = value

    def _get_variant(self):
        return self._variant.value if isinstance(self._variant, State) else self._variant

    def __setupStates(self):
        if isinstance(self._text, State):
            self._text.valueChanged.connect(self.retranslateUi)

    def _init_ui(self):
        self.setLayout(QHBoxLayout())
        # self.layout().setAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignVCenter)

        self._icon = self._get_icon_by_severity()

        self._frm_left = QWidget()
        self._frm_left.setStyleSheet('background: transparent; border: none;')
        self._frm_left.setLayout(QVBoxLayout())
        self._frm_left.layout().setContentsMargins(0,0,0,0)
        self._frm_left.layout().addWidget(self._icon)
        self._frm_left.layout().setAlignment(Qt.AlignmentFlag.AlignTop if not self._action else Qt.AlignmentFlag.AlignCenter)
        self._frm_left.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)

        self.layout().addWidget(self._frm_left)

        self.frm_content = QWidget()
        self.frm_content.setStyleSheet('background: transparent; border: none;')
        self.vlo_frm_content = QVBoxLayout(self.frm_content)
        self.frm_content.setLayout(self.vlo_frm_content)
        self.vlo_frm_content.setContentsMargins(0,0,0,0)
        if self._alertTitle:
            self._lbl_alert_title = QLabel(self)
            self._lbl_alert_title.setObjectName("PyAlertTitle")
            self.vlo_frm_content.addWidget(self._lbl_alert_title)

        self._lbl_alert_text = QLabel(self)
        self._lbl_alert_text.setObjectName("PyAlertContent")
        self.vlo_frm_content.addWidget(self._lbl_alert_text)

        self.layout().addWidget(self.frm_content)

        self._frm_right = QWidget()
        self._frm_right.setStyleSheet('background: transparent; border: none;')
        self._frm_right.setLayout(QVBoxLayout())
        self._frm_right.layout().setContentsMargins(0,0,0,0)
        self._frm_right.layout().setAlignment(Qt.AlignmentFlag.AlignTop)
        self._frm_right.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
        self.layout().addWidget(self._frm_right)

        if self._action:
            self._frm_right_action = QWidget()
            self._frm_right_action.setStyleSheet('background: transparent; border: none;')
            self._frm_right_action.setLayout(QHBoxLayout())
            self._frm_right_action.layout().setContentsMargins(0,0,0,0)
            self._frm_right_action.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
            self._frm_right.layout().addWidget(self._frm_right_action)
            if isinstance(self._action, list):
                for widget in self._action:
                    self._frm_right_action.layout().addWidget(widget)
            else:
                self._frm_right_action.layout().addWidget(self._action)

        if self._onClose:
            self._btn_close = QToolButton()
            self._btn_close.setCursor(Qt.PointingHandCursor)
            self._btn_close.clicked.connect(self._onClose)

            self._frm_right.layout().addWidget(self._btn_close)


        # init theme & language
        i18n.langChanged.connect(self.retranslateUi)
        self.retranslateUi()

        self.slot_set_stylesheet()
        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
        self.destroyed.connect(self._on_destroyed)
        
        self.__setupStates()

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()

    def retranslateUi(self):
        if self._lbl_alert_title:
            self._lbl_alert_title.setText(getTranslatedText(self._alertTitle))
        if self._lbl_alert_text:
            self._lbl_alert_text.setText(getTranslatedText(self._text))
        # if isinstance(self._alertTitle, Callable):
        #     self._lbl_alert_title.setText(translate(self._alertTitle).capitalize())
        # else:
        #     if self._lbl_alert_title:
        #         self._lbl_alert_title.setText(self._alertTitle.capitalize())

        # if isinstance(self._text, Callable):
        #     self._lbl_alert_text.setText(translate(self._text))
        # else:
        #     self._lbl_alert_text.setText(self._text)

    def _set_stylesheet(self, _theme=None):
        theme = useTheme()
        component_styles = theme.components

        self._component_name = f"PyAlert{self._variant.capitalize()}"

        # styles
        PyAlert_root = component_styles[self._component_name].get("styles")["root"][self._color]
        PyAlert_root_color = PyAlert_root['color']
        PyAlert_root_qss = get_qss_style(PyAlert_root)

        PyAlertTitle_root = component_styles["PyAlertTitle"].get("styles")["root"]
        PyAlertTitle_root_qss = get_qss_style(PyAlertTitle_root[self._color])

        # override
        PyAlert_root_qss_override = ""
        PyAlert_root_color_override = None
        if component_styles[self._component_name].get("styleOverrides"):
            PyAlert_root_override = component_styles[self._component_name].get("styleOverrides")["root"][self._color]
            PyAlert_root_color_override = PyAlert_root_override['color']
            PyAlert_root_qss_override = get_qss_style(PyAlert_root_override)

        PyAlertTitle_root_qss_override = ""
        if component_styles[self._component_name].get("styleOverrides"):
            PyAlertTitle_root_override = component_styles["PyAlertTitle"].get("styleOverrides")["root"]
            PyAlertTitle_root_qss_override = get_qss_style(PyAlertTitle_root_override[self._color])

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx


        # if self._lbl_alert_title:
        #     stylesheet = f"""
        #         #PyAlertTitle {{
        #             {PyAlertTitle_root_qss}
        #             {PyAlertTitle_root_qss_override}
        #             {sx_qss}
        #         }}
        #     """
        #     print('stylesheet_________', stylesheet)
        #     self._lbl_alert_title.setStyleSheet(stylesheet)

        # self._lbl_alert_text.setStyleSheet(f"""
        #     #PyAlertContent {{
        #         color: {PyAlert_root_color_override or PyAlert_root_color};
        #     }}
        # """)

        self.setStyleSheet(f"""
            #{self.objectName()} {{
                {PyAlert_root_qss}
                {PyAlert_root_qss_override}
            }}

            #PyAlertTitle {{
                {PyAlertTitle_root_qss}
                {PyAlertTitle_root_qss_override}
            }}

            #PyAlertContent {{
                color: {PyAlert_root_color_override or PyAlert_root_color};
            }}

            {sx_qss}

        """)


        # icon && tool button
        if self._btn_close:
            self._btn_close.setIcon(FluentIconBase().icon_(path=QTMUI_ASSETS.ICONS.CLOSE, color=PyAlert_root_color))
            self._btn_close.setStyleSheet(self._btn_close.styleSheet() + f"""
                QToolButton {{
                    background-color: transparent;
                    border: none;
                    padding: 2px;
                    border-radius: 8px;
                }}
                QToolButton:hover {{
                    background-color: {alpha(PyAlert_root_color_override or PyAlert_root_color, 0.36)};
                }}
            """)

            self._btn_close.setFixedSize(18, 18)

        # print('alert__icon.change__colo__')


    def _map_severity_to_color(self, severity: str) -> str:
        """Map mức độ severity sang màu mặc định."""
        color_mapping = {
            'error': 'red',
            'info': 'blue',
            'success': 'green',
            'warning': 'orange'
        }
        return color_mapping.get(severity, 'success')

    def _get_icon_by_severity(self) -> PyToolButton:
        """Lấy icon mặc định dựa vào mức độ severity."""
        default_icons = {
            'error': QTMUI_ASSETS.ICONS.ALERT.ERROR,
            'info': QTMUI_ASSETS.ICONS.ALERT.INFO,
            'success': QTMUI_ASSETS.ICONS.ALERT.SUCCESS,
            'warning': QTMUI_ASSETS.ICONS.ALERT.WARNING
        }
        return PyToolButton(icon=default_icons.get(self._severity, QTMUI_ASSETS.ICONS.ALERT.INFO))

    def _default_icon_mapping(self) -> dict:
        """Trả về mapping mặc định giữa severity và icon."""
        return {
            'error': 'ErrorOutlinedIcon',
            'info': 'InfoOutlinedIcon',
            'success': 'SuccessOutlinedIcon',
            'warning': 'WarningOutlinedIcon',
        }

    def close(self, event: Any) -> None:
        """Gọi callback onClose nếu được cung cấp."""
        if self._onClose:
            self._onClose(event)

    def render(self) -> dict:
        """Render component Alert thành một đối tượng có thể sử dụng."""
        return {
            'action': self._action,
            'children': self._children,
            'classes': self._classes,
            'closeText': self._closeText,
            'color': self._color,
            'components': self._components,
            'componentsProps': self._componentsProps,
            'icon': self._icon,
            'iconMapping': self._iconMapping,
            'onClose': self._onClose,
            'role': self._role,
            'severity': self._severity,
            'slotProps': self._slotProps,
            'slots': self._slots,
            'sx': self._sx,
            'variant': self._variant,
        }




# Test module for win32timezone

import doctest
import unittest

import win32timezone


class Win32TimeZoneTest(unittest.TestCase):
    def testWin32TZ(self):
        failed, total = doctest.testmod(win32timezone, verbose=False)
        self.assertFalse(failed)


if __name__ == "__main__":
    unittest.main()



# coding:utf-8
from typing import Callable
from PySide6.QtCore import QEasingCurve, QPropertyAnimation, Qt, QEvent
from PySide6.QtGui import QColor, QResizeEvent
from PySide6.QtWidgets import (QDialog, QGraphicsDropShadowEffect, QApplication, QMainWindow,
                             QGraphicsOpacityEffect, QHBoxLayout, QWidget, QFrame)

from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style


class DialogBase(QDialog):
    """ Dialog box base class with a mask """

    def __init__(self, parent=None, *args, **kwargs):
        super().__init__()

        self._parent = parent
        self._sx = kwargs.get("sx")

        self._init_ui()



    def _init_ui(self):
        self._hBoxLayout = QHBoxLayout(self)
        self.windowMask = QWidget(self)
        self.windowMask.setObjectName('windowMask')

        # dialog box in the center of mask, all widgets take it as parent
        self.widget = QFrame(self, objectName='centerWidget')
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        if self._parent:
            self.setGeometry(0, 0, self._parent.width(), self._parent.height())
        elif QApplication.instance().mainWindow:
            self.setGeometry(0, 0, QApplication.instance().mainWindow.width(), QApplication.instance().mainWindow.height())


        c = 0 if useTheme().palette.mode == "dark" else 255
        self.windowMask.resize(self.size())
        self.windowMask.setStyleSheet(f'''
            #windowMask{{
                background:rgba({c}, {c}, {c}, 0.6);
            }}
        ''')

        self._hBoxLayout.addWidget(self.widget)
        
        self.setShadowEffect()

        self.window().installEventFilter(self)

        # self._set_stylesheet()

    def _set_stylesheet(self, component_styled=None):
        
        self.theme = useTheme()

        ownerState = {}

        if not component_styled:
            component_styled = self.theme.components

        c = 0 if self.theme.palette.mode == "dark" else 255

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        # self.windowMask.setStyleSheet(f'''
        #     #windowMask{{
        #         background:rgba({c}, {c}, {c}, 0.6);
        #     }}

        #     {sx_qss}

        # ''')

        self.windowMask.setStyleSheet(f'''
            #windowMask{{
                background:red;
            }}

            {sx_qss}

        ''')
        
        self.setShadowEffect()


    def setShadowEffect(self, blurRadius=60, offset=(0, 10), color=QColor(0, 0, 0, 100)):
        """ add shadow to dialog """
        shadowEffect = QGraphicsDropShadowEffect(self.widget)
        shadowEffect.setBlurRadius(blurRadius)
        shadowEffect.setOffset(*offset)
        shadowEffect.setColor(color)
        self.widget.setGraphicsEffect(None)
        self.widget.setGraphicsEffect(shadowEffect)

    def setMaskColor(self, color: QColor):
        """ set the color of mask """
        self.windowMask.setStyleSheet(f'''
            #windowMask{{
                background: rgba({color.red()}, {color.blue()}, {color.green()}, {color.alpha()});
            }}
        ''')

    def showEvent(self, e):
        """ fade in """
        opacityEffect = QGraphicsOpacityEffect(self)
        self.setGraphicsEffect(opacityEffect)
        opacityAni = QPropertyAnimation(opacityEffect, b'opacity', self)
        opacityAni.setStartValue(0)
        opacityAni.setEndValue(1)
        opacityAni.setDuration(200)
        opacityAni.setEasingCurve(QEasingCurve.InSine)
        opacityAni.finished.connect(lambda: self.setGraphicsEffect(None))
        opacityAni.start()
        super().showEvent(e)

    def done(self, code):
        """ fade out """
        self.widget.setGraphicsEffect(None)
        opacityEffect = QGraphicsOpacityEffect(self)
        self.setGraphicsEffect(opacityEffect)
        opacityAni = QPropertyAnimation(opacityEffect, b'opacity', self)
        opacityAni.setStartValue(1)
        opacityAni.setEndValue(0)
        opacityAni.setDuration(100)
        opacityAni.finished.connect(lambda: self._onDone(code))
        opacityAni.start()

    def _onDone(self, code):
        self.setGraphicsEffect(None)
        QDialog.done(self, code)

    def resizeEvent(self, e):
        self.windowMask.resize(self.size())

    def eventFilter(self, obj, e: QEvent):
        if obj is self.window():
            if e.type() == QEvent.Resize:
                re = QResizeEvent(e)
                self.resize(re.size())

        return super().eventFilter(obj, e)


# src/qtmui/hooks/_effect_context.py
from __future__ import annotations
from typing import List, TYPE_CHECKING
from PySide6.QtCore import QObject
import weakref

if TYPE_CHECKING:
    from ._effect_runner import _EffectRunner  # tránh circular import


# Mỗi component sẽ có thuộc tính riêng: __effect_runners__
def _get_effect_runners(component: QObject) -> List[_EffectRunner]:
    """
    Lấy danh sách effect runners của component hiện tại.
    Tự động tạo nếu chưa có.
    """
    attr_name = "__qtmui_effect_runners__"
    if not hasattr(component, attr_name):
        # Dùng list bình thường (không weakref) vì runner tự deleteLater()
        setattr(component, attr_name, [])
    return getattr(component, attr_name)


def register_effect(component: QObject, runner: _EffectRunner) -> None:
    """
    Đăng ký một effect runner vào component hiện tại
    """
    runners = _get_effect_runners(component)
    runners.append(runner)

    # Tự động cleanup khi component bị destroy
    # Chỉ connect 1 lần duy nhất
    if len(runners) == 1:
        def _on_destroyed():
            runners_copy = list(runners)  # tránh modify while iterating
            for r in runners_copy:
                r.destroy()
            runners.clear()

        # Dùng weakref để tránh reference cycle
        weak_component = weakref.ref(component)
        def _safe_cleanup():
            comp = weak_component()
            if comp is not None:
                _on_destroyed()

        component.destroyed.connect(lambda obj: _safe_cleanup())


def cleanup_component_effects(component: QObject) -> None:
    """Gọi thủ công nếu cần (hiếm khi dùng)"""
    runners = getattr(component, "__qtmui_effect_runners__", None)
    if runners is not None:
        for r in list(runners):
            r.destroy()
        runners.clear()


import inspect

def _get_fn_args(fn):
    # Kiểm tra số lượng tham số của callback
    sig = inspect.signature(fn)
    # Tạo đối số mặc định nếu không có
    default_args = {
        k: (v.default if v.default is not inspect.Parameter.empty else None)
        for k, v in sig.parameters.items()
    }
    return default_args



from typing import Callable, Optional, Union
import uuid
from PySide6.QtWidgets import QWidget, QVBoxLayout

from qtmui.hooks import State
from ...material.masonry.masonry import Masonry

from ..controller import Controller
from ..form_control_label import FormControlLabel
from ..box import Box
from ..form_control import FormControl
from ..form_label import FormLabel
from ..radio import  Radio
from ..radio_group import RadioGroup

from ..qss_name import *

class RHFMasonryRadioGroup(QWidget):
    def __init__(
            self,
            name,
            id: str = None,
            value: object = None,
            row: str = "row",
            columns: int = None,
            checked: bool = False,
            isVisible: bool = False,
            label: Optional[Union[str, State, Callable]]=None,
            options: object = None,
            spacing: int = None,
            minWidth: int = None,
            helperText: str = None,
            color: str = "primary",
            isOptionEqualToValue: Optional[Callable] = lambda checked: False,
            ):
        super().__init__()
        self.setLayout(QVBoxLayout())
        # self.layout().setContentsMargins(0,0,0,0)
        self.setMinimumWidth(400)

        self._id = id
        self._name = name
        self._value = value

        self._isVisible = isVisible
        self._selected = False

        if self._id is not None:
            self.setObjectName(id)
        else:
            self.setObjectName(str(uuid.uuid4()))

        self.setStyleSheet('''#{}  {{ {} }}'''.format(self.objectName(), "border: none;background: #ffeaa7;")) # str multi line

        self._row = row
        self._columns = columns
        self._options = options

        self._color = color

        # if isOptionEqualToValue:
        #     self._isOptionEqualToValue = isOptionEqualToValue
        # else:
        #     self._isOptionEqualToValue = lambda checked=False: checked

        self._isOptionEqualToValue = isOptionEqualToValue

        self._stateSignal = None

        field = {}
        control = False
        error = False


        if self._columns is None:
            self._columns = len(options)

        self._labelledby = f"{label}"  if label else ""

        if minWidth:
            self.setMinimumWidth(minWidth)

        self.layout().addWidget(
            Controller(
                control=control,
                render=FormControl(
                    component="fieldset",
                    children=[
                        Box(
                            direction="column",
                            children=[
                                FormLabel(component="legend",  label=label),
                                self.renderRadioGroup(),
                                # FormHelperText(error=error.get("message") if error and hasattr(error, 'message') else helperText)
                            ]
                        )
                    ]
                )
            )
        )

        self.setup_ui()


    def setup_ui(self):
        for item in self._radio_group.findChildren(Radio):
            # item.toggled.connect(lambda checked, value=item._value: self.set_value(value))
            if not item.checked:
                self._value = item._value
                break

        if isinstance(self._isVisible, State):
            self._isVisible.valueChanged.connect(self._set_visible)
            self._selected = self._isVisible.value
            self._set_hide(self._selected)
        else:
            self._selected = self._isVisible
            self._set_hide(self._selected)

    def renderRadioGroup(self):
        self._radio_group = RadioGroup(
            direction="column",
            # hightLight=True,
            ariaLabelledby=self._labelledby,
            row=self._row,
            children=[
                Masonry(
                    # hightLight=True,
                    columns=self._columns,
                    children=[
                        FormControlLabel(
                            # hightLight=True,
                            labelPlacement="end",
                            key=option["value"],
                            value=option["value"],
                            label=option["label"],
                            fullWidth=False,
                            control=Radio(text=option["label"], value=option["value"], color=self._color,  checked=self._isOptionEqualToValue(option)),
                            # label=option["label"],
                            # sx={
                            #     '&:not(:last-of-type)': {
                            #         mb: spacing || 0,
                            #     },
                            #     ...(row && {
                            #         mr: 0,
                            #         '&:not(:last-of-type)': {
                            #         mr: spacing || 2,
                            #         },
                            #     }),
                            # }
                        ) for option in self._options
                    ]
                ) 
            ]
        )
        self.valueChanged = self._radio_group.valueChanged
        self.valueChanged.connect(self.set_value)

        return self._radio_group

    def _set_hide(self, state):
        self._selected = state
        if not state:
            self.hide()

    def _set_visible(self, state):
        self._selected = state
        if state:
            if not self.isVisible():
                print('show___________________________')
                self.show()
        else:
            if self.isVisible():
                print('hide___________________________')
                self.hide()

    def set_value(self, value):
        self._value = value


    # def showEvent(self, event):
    #     self.valueChanged.emit(self._value)


from typing import Callable
from .qss_properties import QssProperties
from qtmui.material.styles import useTheme
from functools import lru_cache


def camel_to_kebab_case(s):
    """
    Chuyển đổi chuỗi từ camelCase sang kebab-case.
    """
    return ''.join(['-' + c.lower() if c.isupper() else c for c in s]).lstrip('-')

def resolve_theme_value(theme, value):
    """
    Giải quyết giá trị theme từ chuỗi dạng 'palette.primary.main', 'text.secondary', 'background.paper', v.v.
    """
    if isinstance(value, str) and value.startswith("palette."):
        parts = value.split(".")
        theme_value = theme

        for part in parts:
            theme_value = getattr(theme_value, part, None)
            if theme_value is None:
                return None  # Trả về None nếu không tìm thấy giá trị

        # print('theme_value_______________', theme_value)
        return theme_value
    return value






def convert_sx_params_to_str(func):
    @wraps(func)
    def wrapper(sx, *args, **kwargs):
        # Nếu params là dict, chuyển nó thành chuỗi JSON (hoặc sử dụng str() nếu phù hợp)
        if isinstance(sx, dict):
            sx = json.dumps(sx)
        elif isinstance(sx, Callable):
            sx = sx()
            sx = json.dumps(sx)
        else:
            if sx == None:
                sx = ""
        return func(sx, *args, **kwargs)
    return wrapper

def convert_sx_params_to_dict(func):
    @wraps(func)
    def wrapper(sx, *args, **kwargs):
        # Nếu params là dict, chuyển nó thành chuỗi JSON (hoặc sử dụng str() nếu phù hợp)
        if isinstance(sx, str) and sx != "":
            try:
                sx = json.loads(sx)
            except Exception as e:
                print('sxxxxxxxxxx_errrrrrrrrrrrrr', type(sx))
        return func(sx, *args, **kwargs)
    return wrapper



@convert_sx_params_to_str
@lru_cache(maxsize=None)
@convert_sx_params_to_dict
def get_qss_style(sx: dict):
    """
    Hàm tạo chuỗi CSS từ dictionary các tham số.
    """
    if isinstance(sx, str):
        return sx
    theme = useTheme()
    css_lines = []

    for key, value in sx.items():
        # Chuyển key từ camelCase sang kebab-case
        css_key = camel_to_kebab_case(key)

        # Kiểm tra nếu key tồn tại trong QssProperties
        if css_key in QssProperties.__dict__.values():
            # Xử lý giá trị theme (palette., text., background.)
            if isinstance(value, str) and (
                value.startswith("text.") 
                or value.startswith("background.")
                or value.startswith("primary.")
                or value.startswith("secondary.")
                or value.startswith("info.")
                or value.startswith("warning.")
                or value.startswith("error.")
                ):
                value = f"palette.{value}"

                # print('value__resolveeeeeeeeeeeee', value)

            value = resolve_theme_value(theme, value)
            # print('value__resolveeeeeeeeeeeee', value)

            # Xử lý margin và padding X/Y thành hai thuộc tính riêng
            if css_key == "mb" or \
                css_key == "display" or \
                css_key == "flex-shrink" or \
                css_key == "flex-grow" or \
                css_key == "align-items" or \
                css_key == "flex-direction" or \
                css_key == "align-self" or \
                css_key == "z-index" or \
                css_key == "display":
                # print('mbbbbbbbbbbbbbbb')
                continue
            if css_key == "marginX" or css_key == "mx":
                css_lines.append(f"margin-left: {value};")
                css_lines.append(f"margin-right: {value};")
            elif css_key == "marginY" or css_key == "my":
                css_lines.append(f"margin-top: {value};")
                css_lines.append(f"margin-bottom: {value};")
            elif css_key == "paddingX" or css_key == "px":
                css_lines.append(f"padding-left: {value};")
                css_lines.append(f"padding-right: {value};")
            elif css_key == "paddingY" or css_key == "py":
                css_lines.append(f"padding-top: {value};")
                css_lines.append(f"padding-bottom: {value};")
            elif css_key == "width" and str(value).find("%") == -1:
                css_lines.append(f"min-width: {value};")
                css_lines.append(f"max-width: {value};")
            elif css_key == "height" and str(value).find("%") == -1:
                css_lines.append(f"min-height: {value};")
                css_lines.append(f"max-height: {value};")
            else:
                css_lines.append(f"{css_key}: {value};")

    # Kết hợp các dòng CSS thành chuỗi
    return "\n    ".join(css_lines)

def _get_sx_qss(sx):
    sx_qss = ""
    if sx:
        if isinstance(sx, dict):
            sx_qss = get_qss_style(sx)
        elif isinstance(sx, Callable):
            sx = sx()
            if isinstance(sx, dict):
                sx_qss = get_qss_style(sx)
            elif isinstance(sx, str):
                sx_qss = sx
        elif isinstance(sx, str) and sx != "":
            sx_qss = sx
    return sx_qss


_get_sx_qss(
    {
        "color": "red", 
        "background-color": lambda theme: theme.palette.background.main
    }
)


from typing import TYPE_CHECKING, Optional

from ....system.color_manipulator import alpha

from .properties_name import *


if TYPE_CHECKING:
    from ..theme_reducer import ThemeState
    from qtmui.material.styles.create_theme.create_palette import Palette

def pagination(_theme):
    # Khởi tạo các thông tin cần thiết
    theme: ThemeState = _theme

    COLORS = ['primary', 'secondary', 'info', 'success', 'warning', 'error']
    lightMode = theme.palette.mode == 'light'
    
    return {
        'PyPagination': {
            'styles': {
                'root': {
                    **{
                        _color: {
                            "item": {
                                "root": {
                                    "slots": {
                                        "selected": {
                                            fontWeight: theme.typography.fontWeightSemiBold,
                                            borderColor: theme.palette.grey._500,
                                            backgroundColor: alpha(theme.palette.grey._500, 0.08)
                                        }
                                    },
                                    "props": {
                                        "filledVariant": {
                                        },
                                        "outlinedVariant": {
                                        },
                                        "softVariant": {
                                            color: getattr(theme.palette, _color).dark if lightMode else getattr(theme.palette, _color).light,
                                            backgroundColor: alpha(getattr(theme.palette, _color).main, 0.08),
                                            "slots": {
                                                "hover": {
                                                    backgroundColor: alpha(getattr(theme.palette, _color).main, 0.16),
                                                }
                                            }
                                        },
                                    }

                                }
                            }
                        }
                        for _color in COLORS
                    },

                    **{
                        "default": {
                            border: "2px solid transparent",
                            borderRadius: "4px",
                            p: "4px",
                            fontSize: theme.typography.button.fontSize,
                            fontWeight: theme.typography.button.fontWeight,
                            "item": {
                                "root": {
                                    "slots": {
                                        "selected": {
                                            fontWeight: theme.typography.fontWeightSemiBold,
                                            borderColor: theme.palette.grey._500,
                                            backgroundColor: alpha(theme.palette.grey._500, 0.08)
                                        }
                                    },
                                    "props": {
                                        "filledVariant": {
                                            color: theme.palette.common.white if lightMode else theme.palette.grey._800,
                                            backgroundColor: theme.palette.text.primary,
                                            "slots": {
                                                "hover": {
                                                    backgroundColor: theme.palette.grey._700 if lightMode else theme.palette.grey._100,
                                                }
                                            }
                                        },
                                        "outlinedVariant": {
                                            borderColor: alpha(theme.palette.grey._500, 0.24)
                                        },
                                        "softVariant": {
                                        }
                                    }

                                }
                            }
                        },
                    },
                },
            },
        },
    }



import uuid
from PySide6.QtWidgets import QSpacerItem, QWidget, QHBoxLayout, QSizePolicy

class HSpacer(QWidget):
    def __init__(
            self, 
            width: int = None,
            expanding: QSizePolicy = None,
            hightLight: bool = False,
            ):
        super().__init__()
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.setObjectName(str(uuid.uuid4()))
        if hightLight:
            self.setStyleSheet('''#{}  {{ {} }}'''.format(self.objectName(), "background: yellow;")) # str multi line
            
        if width is not None:
            self.layout().addItem(QSpacerItem(width, 0, QSizePolicy.Fixed, QSizePolicy.Minimum))
        else:
            self.layout().addItem(QSpacerItem(10, 10, QSizePolicy.Expanding, QSizePolicy.Minimum))

        if expanding:
            self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)



import asyncio
from typing import Optional, List, Callable, Any, Union
from PySide6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QButtonGroup, QPushButton, QSizePolicy
from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QColor

from qtmui.hooks import State

from .button import Button
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.styles import useTheme

class ToggleButton(Button):
    def __init__(
        self,
        icon: Optional[str] = ":/round/resource_qtmui/round/access_time.svg",
        text: Optional[Union[str, State, Callable]] = None,
        value: Optional[object] = None,
        selected: bool = False,
        *args, **kwargs
    ):
        super().__init__(text=text, startIcon=icon, value=value, *args, **kwargs)

        self._selected = selected

        self._init_ui()

    def _init_ui(self):
        self.setCheckable(True)  # Toggle button cần có trạng thái check/uncheck

        """Thiết lập trạng thái được chọn của ToggleButton."""
        if self._selected:
            super().set_selected(True)

    def _set_stylesheet(self, component_styled=None):
        super()._set_stylesheet()
        self.theme = useTheme()

        if not component_styled:
            component_styled = self.theme.components

        ownerState = {
            "size": self._size
        }
        PyToggleButton_root = component_styled["PyToggleButton"].get("styles")["root"](ownerState)[self._color]
        if self._color in ['primary', 'secondary', 'info', 'success', 'warning', 'error']:
            PyToggleButton_root_slot_hover_qss = get_qss_style(PyToggleButton_root["slots"]["hover"])
            PyToggleButton_root_prop_disabled_selected_qss = get_qss_style(PyToggleButton_root["props"]["disabled"]["selected"])
        else:
            PyToggleButton_root_selected_qss = get_qss_style(PyToggleButton_root["selected"])




from qtmui.material.styles import useTheme

theme = useTheme()

FONT_PRIMARY='Public Sans, sans-serif'
FONT_WEIGHT_REGULAR = 400,

GLOBAL_STYLES = f"""
/* /////////////////////////////////////////////////////////////////////////////////////////////////
QWidget */

QWidget {{
    background-color: transparent;
    border: none;
    font-family: "{FONT_PRIMARY}";
    font-weight: {FONT_WEIGHT_REGULAR};
}}

QDialog {{
    background-color: #ffffff;
    border: 1px solid #DFE3E8;
    font-family: "{FONT_PRIMARY}";
    font-weight: {FONT_WEIGHT_REGULAR};
}}

/* /////////////////////////////////////////////////////////////////////////////////////////////////
ScrollBars */

QScrollBar:horizontal {{
    border: none;
    background: rgb(249, 249, 249);
    height: 4px;
    margin: 0px 21px 0 21px;
    border-radius: 0px;
}}

QScrollBar::handle:horizontal {{
    background: rgb(166, 166, 166);
    min-width: 25px;
    border-radius: 0px
}}

QScrollBar::add-line:horizontal {{
    border: none;
    background: transparent;
    width: 20px;
    border-top-right-radius: 0px;
    border-bottom-right-radius: 0px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}}

QScrollBar::sub-line:horizontal {{
    border: none;
    background: transparent;
    width: 20px;
    border-top-left-radius: 0px;
    border-bottom-left-radius: 0px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}}

QScrollBar::up-arrow:horizontal,
QScrollBar::down-arrow:horizontal {{
    background: none;
}}

QScrollBar::add-page:horizontal,
QScrollBar::sub-page:horizontal {{
    background: none;
}}

QScrollBar:vertical {{
    border: none;
    background: rgb(249, 249, 249);
    width: 4px;
    margin: 21px 0 21px 0;
    border-radius: 0px;
}}

QScrollBar::handle:vertical {{
    background: rgb(166, 166, 166);
    min-height: 25px;
    border-radius: 0px;
}}


QScrollBar::add-line:vertical {{
    border: none;
    background: transparent;
    height: 20px;
    border-bottom-left-radius: 0px; 
    border-bottom-right-radius: 0px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}}

QScrollBar::sub-line:vertical {{
    border: none;
    background: transparent;
    height: 20px;
    border-top-left-radius: 0px;
    border-top-right-radius: 0px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}}

QScrollBar::up-arrow:vertical,
QScrollBar::down-arrow:vertical {{
    background: none;
}}

QScrollBar::add-page:vertical,
QScrollBar::sub-page:vertical {{
    background: none;
}}

/* /////////////////////////////////////////////////////////////////////////////////////////////////
QTabWidget

QTabWidget::pane{{
border: 1px solid {theme.palette.background.paper};
top:-1px;
background: {theme.palette.background.paper};
}}

QTabBar::tab{{
background: transparent;
border: 1px solid {theme.palette.background.paper};
padding: 15px;
}}

QTabBar::tab:selected{{
background: {theme.palette.secondary.main};
margin-bottom: -1px;
}}


 */
"""





import asyncio
from functools import lru_cache
import uuid
from typing import Optional, Callable, Any, Dict, Union

from PySide6.QtWidgets import QLineEdit, QLabel
from PySide6.QtCore import Qt, QEvent, QTimer
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.utils.translator import getTranslatedText
from  .py_input_base import MuiInputBase
from ..widget_base import PyWidgetBase

from qtmui.material.styles import useTheme
from qtmui.hooks import State

from qtmui.i18n.use_translation import i18n

class TFLineEditMultiple(QLineEdit, PyWidgetBase):
    def __init__(
        self,
        textField=None,
        label: Optional[Union[str, State, Callable]] = None,
        placeholder: Optional[Union[str, State, Callable]] = None,
        selectedKeys: State = None,
        onChange: Optional[Callable] = None,
        children: Optional[list] = None,  # Hỗ trợ định nghĩa ghi đè hệ thống cũng như các kiểu CSS bổ sung.
        sx: Optional[Union[Dict, Callable, str]] = None,  # Hỗ trợ định nghĩa ghi đè hệ thống cũng như các kiểu CSS bổ sung.
        variant: str = "outlined",
        size: str = "medium",  # Kích thước của thành phần ('medium' hoặc 'small').
        **kwargs
    ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        self._setUpUi(**kwargs)

        self._children = children
        self._label = label
        self._placeholder = placeholder
        self._onChange = onChange
        self._sx = sx
        self._variant = variant
        self._size = size
        self._selectedKeys = selectedKeys
        self._textField = textField
        
        self._init_ui()
        
    def _init_ui(self):
        self.theme = useTheme()
        
        i18n.langChanged.connect(self._set_placer_holder_text)
        self._selectedKeys.valueChanged.connect(self._onSelectedkeysChanged)

        self.placeholder_label = QLabel("", self)
        self.setStyleSheet("min-height: 20px;padding-left: 0px;padding-top:3px;")
        # self.placeholder_label.setStyleSheet(f"color: gray;border: 1px solid green;min-height: 20px;font-size: 13px;font-weight: 600;")
        self.placeholder_label.setStyleSheet(f"color: gray;min-height: 20px;font-size: 13px;font-weight: 600;")
        self.placeholder_label.setAttribute(Qt.WA_TransparentForMouseEvents, True)
        self.textChanged.connect(self._update_placeholder_visibility)
        self.installEventFilter(self)
        self._update_placeholder_visibility()

        if isinstance(self._placeholder, State):
            self._placeholder.valueChanged.connect(self._set_placer_holder_text)
        self._set_placer_holder_text()

        self.textChanged.connect(self._onChange)

    def _set_placer_holder_text(self):
        # nếu chưa chọn gì và cũng không có focus thì hiển thị placeholder là label
        # print('_set_placer_holder_text')
        
        if len(self._selectedKeys.value) == 0 and not self.hasFocus():
            self.placeholder_label.setText(getTranslatedText(self._label))
            self.setMinimumWidth(self.placeholder_label.sizeHint().width() + 10)
            return
        self.placeholder_label.setText(getTranslatedText(self._placeholder))
        
    def _onSelectedkeysChanged(self):
        print('_onSelectedkeysChanged', self._selectedKeys.value, self._placeholder)

        self.placeholder_label.setText(getTranslatedText(self._placeholder))
        QTimer.singleShot(0, lambda: self.setFocus(Qt.OtherFocusReason))
        

    def enterEvent(self, event):
        PyWidgetBase.enterEvent(self, event)
        super().enterEvent(event)
    
    def leaveEvent(self, event):
        PyWidgetBase.leaveEvent(self, event)
        super().leaveEvent(event)
    
    def mousePressEvent(self, event):
        PyWidgetBase.mousePressEvent(self, event)
        super().mousePressEvent(event)
    
    def mouseReleaseEvent(self, event):
        PyWidgetBase.mouseReleaseEvent(self, event)
        super().mouseReleaseEvent(event)
    
    def focusInEvent(self, event) -> None:
        self.placeholder_label.setText(getTranslatedText(self._placeholder))
        PyWidgetBase.focusInEvent(self, event)
        super().focusInEvent(event)

    def focusOutEvent(self, event) -> None:
        if self.text() == "":
            self._set_placer_holder_text()
        PyWidgetBase.focusOutEvent(self, event)
        super().focusOutEvent(event)
    
    
    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.placeholder_label.resize(self.size())

    def _update_placeholder_visibility(self):
        """Ẩn label khi có text, hiện khi trống."""
        self.placeholder_label.setVisible(self.text() == "")

    def eventFilter(self, obj, event):
        """Không ẩn placeholder khi focus vào."""
        if event.type() in (QEvent.FocusIn, QEvent.FocusOut):
            self._update_placeholder_visibility()
        return super().eventFilter(obj, event)


from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

from .properties_name import *
from ....system.color_manipulator import alpha


def table(_theme) -> Dict:
    theme: ThemeState = _theme
    lightMode = theme.palette.mode == 'light'

    return {
        "PyTableWidget": {
            "styles": {
                "root": f"""
                    QTableView {{
                        background-color: {theme.palette.background.paper};
                        padding: 0px;
                        border: none;
                        gridline-color: transparent;
                        color: {theme.palette.text.secondary};
                    }}

                    QTableView::item {{
                        padding-left: 0px;
                        padding-right: 5px;
                    }}

                    QTableView::item:selected {{
                        background-color: {alpha(theme.palette.primary.dark, 0.04)};
                    }}

                    QTableView::item:hover {{
                        background-color: {alpha(theme.palette.primary.dark, 0.08)};
                    }}

                    QTableView::section {{
                        background-color: transparent;
                        max-width: 30px;
                        text-align: left;
                    }}

                    QTableView::horizontalHeader {{
                        background-color: red;

                    }}

                    QTableView::section:horizontal {{
                        background-color: transparent;
                        padding: 0px;
                        border: 1px solid red !important;
                        border-bottom: 1px solid red !important;
                    }}


                    QTableView::section:vertical {{
                        border: 1px solid red;
                    }}

                    QTableView .QScrollBar:horizontal {{
                        border: none;
                        background: {theme.palette.grey._500};
                        min-height: 8px;
                        border-radius: 0px;
                        max-width: 79em;
                    }}

                    QTableView .QHeaderView::section {{
                        background-color: transparent!important;
                    }}
                """,
                "lineEdit": f"""
                    LineEdit, TextEdit, PlainTextEdit, TextBrowser {{
                        background-color: rgba(255, 255, 255, 0.0605);
                        border: 1px solid rgba(255, 255, 255, 0.08);
                        border-bottom: 1px solid rgba(255, 255, 255, 0.5442);
                        border-radius: 5px;
                        /* font: 14px "Segoe UI", "Microsoft YaHei"; */
                        padding: 0px 10px;
                        color: white;
                        selection-background-color: --ThemeColorPrimary;
                        selection-color: black;
                    }}

                    TextEdit, PlainTextEdit, TextBrowser  {{
                        padding: 2px 3px 2px 8px;
                    }}

                    LineEdit:hover, TextEdit:hover, PlainTextEdit:hover, TextBrowser:hover {{
                        background: rgba(255, 255, 255, 0.0837);
                    }}

                    LineEdit:focus[transparent=true] {{
                        background: rgba(30, 30, 30, 0.7);
                        border-bottom: 1px solid rgba(255, 255, 255, 0.08);
                    }}

                    LineEdit[transparent=false]:focus {{
                        background: rgb(31, 31, 31);
                    }}

                    TextEdit:focus, PlainTextEdit:focus, TextBrowser:focus {{
                        border-bottom: 1px solid --ThemeColorPrimary;
                        background-color: rgba(30, 30, 30, 0.7);
                    }}

                    LineEdit:disabled, TextEdit:disabled, PlainTextEdit:disabled, TextBrowser:disabled {{
                        color: rgba(255, 255, 255, 92);
                        background-color: rgba(255, 255, 255, 0.0419);
                        border: 1px solid rgba(255, 255, 255, 0.0698);
                    }}

                    #lineEditButton {{
                        background-color: transparent;
                        border-radius: 4px;
                        margin: 0;
                    }}

                    #lineEditButton:hover {{
                        background-color: rgba(255, 255, 255, 9);
                    }}

                    #lineEditButton:pressed {{
                        background-color: rgba(255, 255, 255, 6);
                    }}

                """
            },
            "styles1": f"""
                QTableView {{
                    background: transparent;
                    outline: none;
                    border: none;
                    /* font: 13px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC'; */
                    selection-background-color: transparent;
                    alternate-background-color: transparent;
                }}

                QTableView[isBorderVisible=true] {{
                    border: 1px solid rgba(255, 255, 255, 21);
                }}

                QTableView::item {{
                    background: transparent;
                    border: 0px;
                    padding-left: 16px;
                    padding-right: 16px;
                    height: 35px;
                }}

                QTableView::indicator {{
                    width: 18px;
                    height: 18px;
                    border-radius: 5px;
                    border: none;
                    background-color: transparent;
                }}


                QHeaderView {{
                    background-color: transparent;
                }}

                QHeaderView::section {{
                    background-color: transparent;
                    color: rgb(203, 203, 203);
                    padding-left: 5px;
                    padding-right: 5px;
                    border: 1px solid rgba(255, 255, 255, 21);
                    font: 13px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                }}

                QHeaderView::section:horizontal {{
                    border-left: none;
                    height: 33px;
                }}

                QTableView[isBorderVisible=true] QHeaderView::section:horizontal {{
                    border-top: none;
                }}

                QHeaderView::section:horizontal:last {{
                    border-right: none;
                }}

                QHeaderView::section:vertical {{
                    border-top: none;
                }}

                QHeaderView::section:checked {{
                    background-color: transparent;
                }}

                QHeaderView::down-arrow {{
                    subcontrol-origin: padding;
                    subcontrol-position: center right;
                    margin-right: 6px;
                    image: url(:/qfluentwidgets/images/table_view/Down_white.svg);
                }}

                QHeaderView::up-arrow {{
                    subcontrol-origin: padding;
                    subcontrol-position: center right;
                    margin-right: 6px;
                    image: url(:/qfluentwidgets/images/table_view/Up_white.svg);
                }}

                QTableCornerButton::section {{
                    background-color: transparent;
                    border: 1px solid rgba(255, 255, 255, 21);
                }}

                QTableCornerButton::section:pressed {{
                    background-color: rgba(255, 255, 255, 16);
                }}
            """,
        },
    }



from PySide6.QtCore import Qt, QSize

from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.styles.create_theme.create_palette import PaletteColor

from ..py_tool_button.py_tool_button import Iconify
from .button import Button
from ...common.icon import FluentIconBase

class IconButton(Button):
    """
    LoadingButton
    Base Button

    Args:
        loading?: True | False
        loadingPosition?: "start" | "center" | "end"
        loadingIndicator?: "Loading..." | any str

    Returns:
        new instance of LoadingButton
    """
    def __init__(self,
                icon: str,
                edge: str = "end",
                margin: int = 9,
                color: str = "inherit",
                whileTap: str = "tap",
                whileHover: str = "hover",
                *args, **kwargs
                ):
        super().__init__(color=color, startIcon=icon, *args, **kwargs)

        self._icon = icon
        self._color = color
        self._whileTap = whileTap
        self._whileHover = whileHover

        # self.layout().addWidget(icon)
        # self.layout().setContentsMargins(0,0,9,0)
        stylesheet = f"""
            QPushButton {{
                background-color: transparent;
            }}
        """
        # self.setStyleSheet(self.styleSheet() + stylesheet)
        # self.autorun_set_icon()

    def set_icon(self, theme):
        pass
        is_light_mode = theme.palette.mode == 'light'
        if self._color in ['primary', 'secondary', 'info', 'success', 'warning', 'error']:
            palette_color: PaletteColor = getattr(theme.palette, self._color)
            self._icon_color = palette_color.light if is_light_mode else palette_color.dark
        else: # inherit
            self._icon_color = theme.palette.grey._800 if is_light_mode else theme.palette.common.white
        # print("self._color________", self._icon_color)
        self.setIcon(FluentIconBase().icon_(path=self._icon, color=self._icon_color))
        self.setIconSize(QSize(18, 18))




from typing import Callable, Optional, Sequence
from PySide6.QtCore import Qt, Property, Slot, Signal
from PySide6.QtWidgets import QWidget, QVBoxLayout, QFrame, QLabel

from qtmui.material.styles import useTheme
from qtmui.hooks import useState

from qtmui.hooks import State
from ...material.controller import Controller
from ..editor import Editor

class RHFEditor(QFrame):
    valueChanged = Signal(object)
    def __init__(
            self,
            name,
            max: int = 5,
            value: int = 3,
            onChange: Optional[Callable] = None,
            simple: Optional[bool] = None,
            helperText: str = None,
            **kwargs
            ):
        super().__init__()
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(0)

        self._name = name
        self._value = None
        
        self._state, self._setState = useState(None)
        
        self._onChange = onChange
        self._helperText = helperText

        self._stateSignal = None

        control = False

        self.layout().addWidget(
            Controller(
            name=name,
            control=control,
            render=Editor(onChange=self._set_value, simple=simple, **kwargs)
            )
        )

        self.lbl_helper_text = QLabel(self)
        self.lbl_helper_text.setStyleSheet(f'''
            padding-left: 8px;
            {useTheme().typography.caption.to_qss_props()}
            font-size: 11px;
        ''')
        self.lbl_helper_text.setText("")
        self.layout().addWidget(self.lbl_helper_text)

        # self._complete.valueChanged.connect(self.set_value)

        # if self._onValueChanged is not None:
        #     self.valueChanged.connect(self._onValueChanged)


    def _set_value(self, value=None):
        self._value = value
        if self._value == []:
            self._value = None
        print(f"RHFRating {self._name}: {self._value}")
        self.valueChanged.emit(value)

        if self._onChange:
            self._onChange(value)
            
        self._setState(self._value)

    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        theme = useTheme()
        if self._name == state.get("field"):
            if state.get("error"):
                # self._complete._inputField._set_slot({"slot": "error", "message": state.get("error_message")})
                self.lbl_helper_text.setText(str(state.get("error_message")[0]))
                self.lbl_helper_text.setStyleSheet(f'''
                    padding-left: 8px;
                    font-size: {theme.typography.caption.fontSize};
                    font-weight: {theme.typography.caption.fontWeight};
                    color: {theme.palette.error.main};
                ''')
                if not self.lbl_helper_text.isVisible():
                    self.lbl_helper_text.show()
            else:
                # self._complete._inputField._set_slot({"slot": "valid"})
                self.lbl_helper_text.hide()





from .use_state import useState, State
from dataclasses import dataclass

from typing import Callable, Dict
from PySide6.QtWidgets import QApplication, QPushButton, QVBoxLayout, QMainWindow, QWidget, QLineEdit
from PySide6.QtCore import QObject, Property, Signal



# @dataclass
# class UseFormType:
#   state: State
#   handleSubmit: Callable
#   setValue: Callable
#   formState: Callable
# #   formState: dict
#   control: dict
#   watch: Callable -> State...
#   reset: Callable
#   resolver: Callable

@dataclass
class UseFormType:
    state: State
    handleSubmit: Callable[..., any]
    setValue: Callable[..., any]
    formState: Callable[..., any]
    control: dict
    watch: Callable[..., dict]
    reset: Callable[..., any]
    resolver: Callable[..., any]

class UseForm(QObject): # 
    controlChanged = Signal(object)
    stateChanged = Signal(object)

    def __init__(self, resolver=None, values: dict=None):
        super().__init__()
        if isinstance(values, State):
            self.state: State = values
            self.data, self.setData = useState(self.state.value)
        elif isinstance(values, dict):
            self.state, _ = useState(values)
            self.data, self.setData = useState(self.state.value)

        self.resolver = resolver

    def handleSubmit(self, *args, **kwargs):
        self.setData(True)

    def setValue(self):
        """Đặt lại trạng thái mở về None."""
        self.setData(False)

    def watch(self):
        """In ra fromState."""
        # print(self.state)
        return self.state.value

    def reset(self):
        """In ra fromState."""
        self.setData({})

    # def getFormState(self):
    #     return self.formState

    # def setFormState(self, value):
    #     self.formState = value
    #     # self.stateChanged.emit(self.formState)

    def getControl(self):
        return self.control

    def formState(self):
        return self.state
    
    def control(self):
        return self.state

    def setControl(self, value):
        self.control = value
        self.controlChanged.emit(self.control)

    # formState = Property(object, getFormState, setFormState, notify=stateChanged)
    # control = Property(object, getControl, setControl, notify=controlChanged)
 
def useForm(resolver: Callable=None, values: object=None)->UseFormType:
    """
    Hàm này trả về một instance của Popover,
    tương tự như cách bạn sử dụng hook trong React.
    """
    return UseForm(resolver, values)



# quickEdit = useBoolean(False)





import re
from typing import Any, Dict, Optional, Tuple

from PySide6.QtWidgets import QFrame
from PySide6.QtCore import QSize, Qt, QRectF, QPointF
from PySide6.QtGui import (
    QPainter, QColor, QPen, QPixmap,
    QBrush, QLinearGradient, QPainterPath
)




# =============================================================================
# Helper: StyleDecorator (border + radius + background + cache pixmap)
# =============================================================================

class StyleDecorator:
    """
    Helper xử lý style dạng sx cho QWidget:
      - Parse sx 1 lần (tránh parse/alloc trong paintEvent)
      - Render ra pixmap cache theo (size, dpr, styleKey)
      - Paint: chỉ drawPixmap (nhẹ)

    Hỗ trợ:
      - border (4 cạnh khác nhau hoặc đồng nhất)
      - border-radius (uniform hoặc theo từng góc)
      - background: color / linear-gradient(...)
    """

    def __init__(self, owner: QFrame, sx: Optional[Dict[str, Any]] = None):
        self._owner = owner
        self._sx: Dict[str, Any] = sx or {}

        self._border_spec: Dict[str, Dict[str, Any]] = {}
        self._radius_spec: Dict[str, float] = {}
        self._bg_spec: Dict[str, Any] = {}

        self._cache_size: Optional[Tuple[int, int]] = None
        self._cache_dpr: Optional[float] = None
        self._cache_key: Optional[str] = None
        self._pixmap: Optional[QPixmap] = None

        self.set_sx(self._sx)

    # ------------------------- Public API -------------------------

    def set_sx(self, sx: Dict[str, Any]):
        self._sx = sx or {}
        self._border_spec = self._parse_border_sx(self._sx)
        self._radius_spec = self._parse_radius_sx(self._sx)
        self._bg_spec = self._parse_background_sx(self._sx)
        self.invalidate_cache()

    def invalidate_cache(self):
        self._cache_size = None
        self._cache_dpr = None
        self._cache_key = None
        self._pixmap = None

    def paint(self, painter: QPainter, rect_size: QSize, dpr: float):
        pm = self._get_pixmap(rect_size, dpr)
        if pm is None or pm.isNull():
            return
        painter.drawPixmap(0, 0, pm)

    # ------------------------- Enum helpers (PySide6-safe) -------------------------

    def _enum_to_int(self, v) -> int:
        try:
            if isinstance(v, int):
                return v
            if hasattr(v, "value"):
                return int(v.value)
            return int(v)
        except Exception:
            return 0

    # ------------------------- Parsing utils -------------------------

    def _parse_px(self, s: str) -> float:
        if not isinstance(s, str):
            return 0.0
        m = re.search(r"(-?\d+(?:\.\d+)?)\s*px", s.strip(), re.I)
        if not m:
            m = re.search(r"(-?\d+(?:\.\d+)?)", s.strip())
        if not m:
            return 0.0
        try:
            return max(0.0, float(m.group(1)))
        except Exception:
            return 0.0

    def _parse_color(self, v: Any) -> QColor:
        if v is None:
            return QColor(0, 0, 0, 0)
        if isinstance(v, QColor):
            return v
        if isinstance(v, (tuple, list)) and len(v) in (3, 4):
            r, g, b = int(v[0]), int(v[1]), int(v[2])
            a = int(v[3]) if len(v) == 4 else 255
            return QColor(r, g, b, max(0, min(255, a)))

        if not isinstance(v, str):
            return QColor(0, 0, 0, 0)

        s = v.strip()

        m = re.search(
            r"rgba?\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)(?:\s*,\s*([0-9.]+)\s*)?\)",
            s, re.I
        )
        if m:
            r = int(m.group(1))
            g = int(m.group(2))
            b = int(m.group(3))
            a_raw = m.group(4)
            if a_raw is None:
                a = 255
            else:
                a_f = float(a_raw)
                a = int(round(a_f * 255)) if a_f <= 1.0 else int(round(a_f))
                a = max(0, min(255, a))
            return QColor(r, g, b, a)

        if s.startswith("#"):
            c = QColor(s)
            if c.isValid():
                return c

        c = QColor(s)
        if c.isValid():
            return c

        return QColor(0, 0, 0, 0)

    def _parse_pen_style_int(self, s: str) -> int:
        ss = (s or "").lower()
        if "dashed" in ss:
            return self._enum_to_int(Qt.PenStyle.DashLine)
        if "dotted" in ss:
            return self._enum_to_int(Qt.PenStyle.DotLine)
        return self._enum_to_int(Qt.PenStyle.SolidLine)

    # ------------------------- Border parsing -------------------------

    def _parse_border_value(self, v: Any) -> Dict[str, Any]:
        """
        Output: {"w": int, "color": QColor, "style": int(enum_value)}
        """
        solid_i = self._enum_to_int(Qt.PenStyle.SolidLine)

        if v is None:
            return {"w": 0, "color": QColor(0, 0, 0, 0), "style": solid_i}

        if isinstance(v, (int, float)):
            w = max(0, int(round(v)))
            return {"w": w, "color": QColor(0, 0, 0, 255), "style": solid_i}

        if not isinstance(v, str):
            return {"w": 0, "color": QColor(0, 0, 0, 0), "style": solid_i}

        s = v.strip()
        w = int(round(self._parse_px(s)))
        style_i = self._parse_pen_style_int(s)

        m_rgba = re.search(r"rgba?\([^)]+\)", s, re.I)
        if m_rgba:
            color = self._parse_color(m_rgba.group(0))
        else:
            m_hex = re.search(r"(#(?:[0-9a-fA-F]{6}|[0-9a-fA-F]{8}))", s)
            if m_hex:
                color = self._parse_color(m_hex.group(1))
            else:
                color = QColor(0, 0, 0, 255)

        return {"w": w, "color": color, "style": style_i}

    def _parse_border_sx(self, sx: Dict[str, Any]) -> Dict[str, Dict[str, Any]]:
        base = self._parse_border_value(sx.get("border") or sx.get("borderWidth") or sx.get("border-width"))

        def pick(*keys):
            for k in keys:
                if k in sx and sx[k] is not None:
                    return sx[k]
            return None

        left_v = pick("borderLeft", "border-left", "borderLeftWidth", "border-left-width")
        right_v = pick("borderRight", "border-right", "borderRightWidth", "border-right-width")
        top_v = pick("borderTop", "border-top", "borderTopWidth", "border-top-width")
        bottom_v = pick("borderBottom", "border-bottom", "borderBottomWidth", "border-bottom-width")

        left = self._parse_border_value(left_v) if left_v is not None else base
        right = self._parse_border_value(right_v) if right_v is not None else base
        top = self._parse_border_value(top_v) if top_v is not None else base
        bottom = self._parse_border_value(bottom_v) if bottom_v is not None else base

        return {"left": left, "right": right, "top": top, "bottom": bottom}

    # ------------------------- Radius parsing -------------------------

    def _parse_radius_sx(self, sx: Dict[str, Any]) -> Dict[str, float]:
        base = sx.get("borderRadius", sx.get("border-radius", 0))
        base_r = float(base) if isinstance(base, (int, float)) else float(self._parse_px(str(base)))
        base_r = max(0.0, base_r)

        def pick_radius(*keys) -> float:
            for k in keys:
                if k in sx and sx[k] is not None:
                    v = sx[k]
                    if isinstance(v, (int, float)):
                        return max(0.0, float(v))
                    return max(0.0, float(self._parse_px(str(v))))
            return base_r

        tl = pick_radius("borderTopLeftRadius", "border-top-left-radius")
        tr = pick_radius("borderTopRightRadius", "border-top-right-radius")
        br = pick_radius("borderBottomRightRadius", "border-bottom-right-radius")
        bl = pick_radius("borderBottomLeftRadius", "border-bottom-left-radius")

        return {"tl": tl, "tr": tr, "br": br, "bl": bl}

    # ------------------------- Background parsing -------------------------

    def _parse_linear_gradient(self, s: str) -> Optional[Dict[str, Any]]:
        s = s.strip()
        if not s.lower().startswith("linear-gradient(") or not s.endswith(")"):
            return None

        inner = s[len("linear-gradient("):-1].strip()
        parts = [p.strip() for p in inner.split(",") if p.strip()]
        if len(parts) < 2:
            return None

        direction = "to bottom"
        colors = parts

        first = parts[0].lower()
        if first.startswith("to ") or first.endswith("deg"):
            direction = parts[0]
            colors = parts[1:]

        if len(colors) < 2:
            return None

        c1 = self._parse_color(colors[0])
        c2 = self._parse_color(colors[1])

        return {"type": "linear-gradient", "dir": direction, "c1": c1, "c2": c2}

    def _parse_background_sx(self, sx: Dict[str, Any]) -> Dict[str, Any]:
        v = sx.get("background", None)
        if v is None:
            v = sx.get("backgroundColor", sx.get("background-color", None))

        if v is None:
            return {"type": "none"}

        if isinstance(v, str):
            grad = self._parse_linear_gradient(v)
            if grad:
                return grad

        return {"type": "color", "color": self._parse_color(v)}

    # ------------------------- Cache key -------------------------

    def _style_key(self) -> str:
        b = self._border_spec
        r = self._radius_spec
        bg = self._bg_spec

        def border_one(side: str) -> str:
            s = b.get(side, {})
            c: QColor = s.get("color", QColor(0, 0, 0, 0))
            style_i = self._enum_to_int(s.get("style", self._enum_to_int(Qt.PenStyle.SolidLine)))
            return f"{side}:{int(s.get('w', 0) or 0)}:{style_i}:{c.rgba()}"

        b_key = "|".join([border_one("left"), border_one("top"), border_one("right"), border_one("bottom")])
        r_key = f"r:{r.get('tl',0):.2f},{r.get('tr',0):.2f},{r.get('br',0):.2f},{r.get('bl',0):.2f}"

        if bg.get("type") == "none":
            bg_key = "bg:none"
        elif bg.get("type") == "color":
            c: QColor = bg.get("color", QColor(0, 0, 0, 0))
            bg_key = f"bg:color:{c.rgba()}"
        else:
            c1: QColor = bg.get("c1", QColor(0, 0, 0, 0))
            c2: QColor = bg.get("c2", QColor(0, 0, 0, 0))
            bg_key = f"bg:grad:{bg.get('dir','')}:{c1.rgba()}:{c2.rgba()}"

        return f"{b_key}||{r_key}||{bg_key}"

    # ------------------------- Pixmap render -------------------------

    def _get_pixmap(self, size: QSize, dpr: float) -> Optional[QPixmap]:
        w = max(0, int(size.width()))
        h = max(0, int(size.height()))
        if w <= 0 or h <= 0:
            return None

        key = self._style_key()

        if (
            self._pixmap is not None
            and self._cache_size == (w, h)
            and self._cache_dpr == dpr
            and self._cache_key == key
        ):
            return self._pixmap

        pm = QPixmap(int(w * dpr), int(h * dpr))
        pm.setDevicePixelRatio(dpr)
        pm.fill(Qt.GlobalColor.transparent)

        p = QPainter(pm)
        p.setRenderHint(QPainter.RenderHint.Antialiasing, True)

        self._render(p, float(w), float(h))

        p.end()

        self._pixmap = pm
        self._cache_size = (w, h)
        self._cache_dpr = dpr
        self._cache_key = key
        return pm

    def _render(self, p: QPainter, w: float, h: float):
        self._render_background(p, w, h)
        self._render_border(p, w, h)

    def _rounded_path(self, rect: QRectF, r: Dict[str, float]) -> QPainterPath:
        tl = float(r.get("tl", 0.0))
        tr = float(r.get("tr", 0.0))
        br = float(r.get("br", 0.0))
        bl = float(r.get("bl", 0.0))

        max_rx = rect.width() / 2.0
        max_ry = rect.height() / 2.0
        tl = min(tl, max_rx, max_ry)
        tr = min(tr, max_rx, max_ry)
        br = min(br, max_rx, max_ry)
        bl = min(bl, max_rx, max_ry)

        x, y, rw, rh = rect.x(), rect.y(), rect.width(), rect.height()
        path = QPainterPath()

        path.moveTo(x + tl, y)
        path.lineTo(x + rw - tr, y)
        if tr > 0:
            path.quadTo(x + rw, y, x + rw, y + tr)
        else:
            path.lineTo(x + rw, y)

        path.lineTo(x + rw, y + rh - br)
        if br > 0:
            path.quadTo(x + rw, y + rh, x + rw - br, y + rh)
        else:
            path.lineTo(x + rw, y + rh)

        path.lineTo(x + bl, y + rh)
        if bl > 0:
            path.quadTo(x, y + rh, x, y + rh - bl)
        else:
            path.lineTo(x, y + rh)

        path.lineTo(x, y + tl)
        if tl > 0:
            path.quadTo(x, y, x + tl, y)
        else:
            path.lineTo(x, y)

        path.closeSubpath()
        return path

    def _render_background(self, p: QPainter, w: float, h: float):
        bg = self._bg_spec
        if bg.get("type") == "none":
            return

        rect = QRectF(0.0, 0.0, w, h)
        r = self._radius_spec
        has_radius = any((r.get("tl", 0), r.get("tr", 0), r.get("br", 0), r.get("bl", 0)))

        if has_radius:
            path = self._rounded_path(rect, r)
            p.save()
            p.setClipPath(path)

        if bg.get("type") == "color":
            c: QColor = bg.get("color", QColor(0, 0, 0, 0))
            p.fillRect(rect, QBrush(c))
        elif bg.get("type") == "linear-gradient":
            grad = self._build_linear_gradient(w, h, bg.get("dir", "to bottom"), bg.get("c1"), bg.get("c2"))
            p.fillRect(rect, QBrush(grad))

        if has_radius:
            p.restore()

    def _build_linear_gradient(self, w: float, h: float, direction: str, c1: QColor, c2: QColor) -> QLinearGradient:
        direction = (direction or "").strip().lower()
        c1 = c1 if isinstance(c1, QColor) else QColor(0, 0, 0, 0)
        c2 = c2 if isinstance(c2, QColor) else QColor(0, 0, 0, 0)

        x1, y1, x2, y2 = 0.0, 0.0, 0.0, h

        if direction.startswith("to "):
            if "right" in direction:
                x1, y1, x2, y2 = 0.0, 0.0, w, 0.0
            elif "left" in direction:
                x1, y1, x2, y2 = w, 0.0, 0.0, 0.0
            elif "top" in direction:
                x1, y1, x2, y2 = 0.0, h, 0.0, 0.0
            else:
                x1, y1, x2, y2 = 0.0, 0.0, 0.0, h
        elif direction.endswith("deg"):
            try:
                import math
                deg = float(direction.replace("deg", "").strip())
                rad = math.radians(deg)
                vx = math.sin(rad)
                vy = -math.cos(rad)
                cx, cy = w / 2.0, h / 2.0
                half = max(w, h) / 2.0
                x1, y1 = cx - vx * half, cy - vy * half
                x2, y2 = cx + vx * half, cy + vy * half
            except Exception:
                x1, y1, x2, y2 = 0.0, 0.0, 0.0, h

        g = QLinearGradient(QPointF(x1, y1), QPointF(x2, y2))
        g.setColorAt(0.0, c1)
        g.setColorAt(1.0, c2)
        return g

    def _border_is_uniform(self) -> bool:
        s = self._border_spec
        a = s["left"]
        for k in ("top", "right", "bottom"):
            b = s[k]
            if int(a.get("w", 0) or 0) != int(b.get("w", 0) or 0):
                return False
            if int(a.get("style", 0) or 0) != int(b.get("style", 0) or 0):
                return False
            ca: QColor = a.get("color", QColor(0, 0, 0, 0))
            cb: QColor = b.get("color", QColor(0, 0, 0, 0))
            if ca.rgba() != cb.rgba():
                return False
        return int(a.get("w", 0) or 0) > 0

    def _render_border(self, p: QPainter, w: float, h: float):
        s = self._border_spec
        lw = int(s["left"].get("w", 0) or 0)
        rw = int(s["right"].get("w", 0) or 0)
        tw = int(s["top"].get("w", 0) or 0)
        bw = int(s["bottom"].get("w", 0) or 0)

        if (lw + rw + tw + bw) <= 0:
            return

        if self._border_is_uniform():
            self._render_uniform_border(p, w, h)
            return

        x_left = lw / 2.0 if lw > 0 else 0.0
        x_right = (w - rw / 2.0) if rw > 0 else w
        y_top = tw / 2.0 if tw > 0 else 0.0
        y_bottom = (h - bw / 2.0) if bw > 0 else h

        def set_pen(side_spec: Dict[str, Any]):
            bw_ = int(side_spec.get("w", 0) or 0)
            col: QColor = side_spec.get("color", QColor(0, 0, 0, 255))
            style_i = int(side_spec.get("style", self._enum_to_int(Qt.PenStyle.SolidLine)) or 0)
            pen = QPen(col, bw_, Qt.PenStyle(style_i))
            pen.setCapStyle(Qt.PenCapStyle.SquareCap)
            p.setPen(pen)

        if tw > 0:
            set_pen(s["top"])
            p.drawLine(QPointF(x_left, y_top), QPointF(x_right, y_top))
        if bw > 0:
            set_pen(s["bottom"])
            p.drawLine(QPointF(x_left, y_bottom), QPointF(x_right, y_bottom))
        if lw > 0:
            set_pen(s["left"])
            p.drawLine(QPointF(x_left, y_top), QPointF(x_left, y_bottom))
        if rw > 0:
            set_pen(s["right"])
            p.drawLine(QPointF(x_right, y_top), QPointF(x_right, y_bottom))

    def _render_uniform_border(self, p: QPainter, w: float, h: float):
        s = self._border_spec["left"]
        bw = int(s.get("w", 0) or 0)
        if bw <= 0:
            return

        col: QColor = s.get("color", QColor(0, 0, 0, 255))
        style_i = int(s.get("style", self._enum_to_int(Qt.PenStyle.SolidLine)) or 0)
        pen = QPen(col, bw, Qt.PenStyle(style_i))
        pen.setCapStyle(Qt.PenCapStyle.SquareCap)
        p.setPen(pen)
        p.setBrush(Qt.BrushStyle.NoBrush)

        inset = bw / 2.0
        rect = QRectF(inset, inset, max(0.0, w - 2 * inset), max(0.0, h - 2 * inset))

        r = self._radius_spec
        has_radius = any((r.get("tl", 0), r.get("tr", 0), r.get("br", 0), r.get("bl", 0)))
        if not has_radius:
            p.drawRect(rect)
            return

        rr = {
            "tl": max(0.0, r.get("tl", 0.0) - inset),
            "tr": max(0.0, r.get("tr", 0.0) - inset),
            "br": max(0.0, r.get("br", 0.0) - inset),
            "bl": max(0.0, r.get("bl", 0.0) - inset),
        }
        path = self._rounded_path(rect, rr)
        p.drawPath(path)




from typing import Callable, Optional, Union
import uuid
from PySide6.QtCore import Qt, Property, Slot
from PySide6.QtWidgets import QWidget, QVBoxLayout, QSizePolicy, QLabel, QHBoxLayout, QFrame

from qtmui.hooks import State

from ..autocomplete.complete import Autocomplete
from qtmui.material.styles import useTheme

from ..qss_name import *

class RHFAutocompleteGroup(QWidget):
    def __init__(
            self,
            label: Optional[Union[str, State, Callable]]=None,
            name: str = None,
            value: object = None,
            onValueChanged: Callable = None,
            direction: str = "row",
            children: list = None,
            ):
        super().__init__()
        
        

        self._name = name
        self._value = value
        self._onValueChanged = onValueChanged

        self._stateSignal = None

        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        self.frm_content = QFrame()
        
        if direction == "row":
            self.lo_frm_content = QHBoxLayout(self.frm_content)
        else: # column
            self.lo_frm_content = QVBoxLayout(self.frm_content)

        self.lo_frm_content.setContentsMargins(0,0,0,0)

        if children is not None:
            if isinstance(children, list):
                for item in children:
                    self.lo_frm_content.addWidget(item)

        self.lbl_helper_text = QLabel(self)
        self.lbl_helper_text.setFixedHeight(14)
        self.lbl_helper_text.setStyleSheet('padding-left: 8px;')
        self.lbl_helper_text.setText("")
        self.layout().addWidget(self.frm_content)
        self.layout().addWidget(self.lbl_helper_text)

        self.setup_ui()

    def setup_ui(self):
        # kiểm tra selected của các RHFAutocomplete, thông báo lỗi nếu tìm thấy có nhiều hơn 1 field có trường này == True
        selected_item_count = 0
        for item in self.findChildren(Autocomplete):
            item.valueChanged.connect(lambda value, item_name=item.objectName(): self._onValueChanged(item_name))
            item.valueChanged.connect(lambda value, item=item: self.on_fields_value_changed(value, item))
            if item._selected:
                self._value = item._value
                selected_item_count += 1
            else:
                item._value = None
                
        # xem lai
        # if selected_item_count > 1:
        #     raise AttributeError("Only one child element of RHFAutocompleteGroup is allowed to have the attribute selected set to True.")
        # if selected_item_count == 0:
        #     raise AttributeError("At least one AutoComplete must have the selected attribute set to True.")

    def unselected_all_field(self):
        for item in self.findChildren(Autocomplete):
            item.selected = False

    def on_fields_value_changed(self, value, item):
        print('item____________',value,  item)
        self._value = value
        self.unselected_all_field()
        item.selected = True

    def set_value(self, value):
        print('show_even  ===> set_value')
        self._value = value

    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        theme = useTheme()
        if self._name == state.get("field"):
            if state.get("error"):
                self.set_state({"status": "invalid", "message": state.get("error_message")})
                self.lbl_helper_text.setText(str(state.get("error_message")))
                self.lbl_helper_text.setStyleSheet(f'padding-left: 8px;color: {theme.palette.error.main};')